"""Detector module class"""

from http import HTTPStatus

from ..common.base import Base
from ..common.config import *

MODULE_NAME = DETECTOR


class Detector:
    def __init__(self, calibration_client,
                 name, identifier, karabo_name, karabo_id_control,
                 flg_available, description=''):
        self.calibration_client = calibration_client
        self.id = None
        self.name = name
        self.identifier = identifier
        self.karabo_name = karabo_name
        self.karabo_id_control = karabo_id_control
        self.flg_available = flg_available
        self.description = description

    def create(self):
        cal_client = self.calibration_client
        response = cal_client.create_detector_api(self.__get_resource())

        Base.cal_debug(MODULE_NAME, CREATE, response)
        res = Base.format_response(
            response, CREATE, HTTPStatus.CREATED, MODULE_NAME
        )

        if res['success']:
            self.id = res['data']['id']

        return res

    def delete(self):
        cal_client = self.calibration_client
        response = cal_client.delete_detector_api(self.id)
        Base.cal_debug(MODULE_NAME, DELETE, response)

        return Base.format_response(
            response, DELETE, HTTPStatus.NO_CONTENT, MODULE_NAME
        )

    def update(self):
        cal_client = self.calibration_client
        response = cal_client.update_detector_api(self.id,
                                                  self.__get_resource())

        Base.cal_debug(MODULE_NAME, UPDATE, response)
        return Base.format_response(
            response, UPDATE, HTTPStatus.OK, MODULE_NAME
        )

    @staticmethod
    def get_by_id(cal_client, detector_id):
        response = cal_client.get_detector_by_id_api(detector_id)

        Base.cal_debug(MODULE_NAME, 'get_by_id', response)
        return Base.format_response(response, GET, HTTPStatus.OK, MODULE_NAME)

    @staticmethod
    def get_all_by_name(cal_client, name, page=DEF_PAGE, page_size=DEF_PAGE_SIZE):
        response = cal_client.get_all_detectors_by_name_api(name, page, page_size)

        Base.cal_debug(MODULE_NAME, 'get_all_by_name', response)
        return Base.format_response(response, GET, HTTPStatus.OK, MODULE_NAME)

    @staticmethod
    def get_all_by_identifier(cal_client, identifier):
        response = cal_client.get_all_detectors_by_identifier_api(identifier)

        Base.cal_debug(MODULE_NAME, 'get_all_by_identifier', response)
        return Base.format_response(response, GET, HTTPStatus.OK, MODULE_NAME)

    @staticmethod
    def get_by_name(cal_client, name):
        res = Detector.get_all_by_name(cal_client, name)

        if res['success']:
            if res['data'] == []:
                resp_data = []
            else:
                resp_data = res['data'][0]

            res = {'success': res['success'],
                   'info': res['info'],
                   'app_info': res['app_info'],
                   'data': resp_data}

        return res

    @staticmethod
    def get_by_identifier(cal_client, identifier):
        res = Detector.get_all_by_identifier(cal_client, identifier)

        if res['success']:
            if res['data'] == []:
                resp_data = []
            else:
                resp_data = res['data'][0]

            res = {'success': res['success'],
                   'info': res['info'],
                   'app_info': res['app_info'],
                   'data': resp_data}

        return res

    def __get_resource(self):
        detector = {
            DETECTOR: {
                'name': self.name,
                'identifier': self.identifier,
                'karabo_name': self.karabo_name,
                'karabo_id_control': self.karabo_id_control,
                'flg_available': self.flg_available,
                'description': self.description
            }
        }

        return detector



"""CalibrationConstantVersion module class"""

from http import HTTPStatus

from ..common.base import Base
from ..common.config import *

MODULE_NAME = CALIBRATION_CONSTANT_VERSION


class CalibrationConstantVersion:
    def __init__(self, calibration_client,
                 name, file_name, path_to_file, data_set_name,
                 calibration_constant_id, physical_detector_unit_id,
                 flg_deployed, flg_good_quality,
                 begin_validity_at, end_validity_at, begin_at,
                 start_idx, end_idx, raw_data_location, report_id,
                 description=''):
        self.calibration_client = calibration_client
        self.id = None
        self.name = name
        self.file_name = file_name
        self.path_to_file = path_to_file
        self.data_set_name = data_set_name
        self.calibration_constant_id = calibration_constant_id
        self.physical_detector_unit_id = physical_detector_unit_id
        self.flg_deployed = flg_deployed
        self.flg_good_quality = flg_good_quality
        self.begin_validity_at = begin_validity_at
        self.end_validity_at = end_validity_at
        self.begin_at = begin_at
        self.start_idx = start_idx
        self.end_idx = end_idx
        self.raw_data_location = raw_data_location
        self.report_id = report_id
        self.description = description

    def create(self):
        cal_client = self.calibration_client
        resp = cal_client.create_calibration_constant_version_api(
            self.__get_resource())

        Base.cal_debug(MODULE_NAME, CREATE, resp)
        res = Base.format_response(
            resp, CREATE, HTTPStatus.CREATED, MODULE_NAME
        )

        if res['success']:
            self.id = res['data']['id']

        return res

    #
    # Delete CalibrationConstantVersion instances is not allowed by the server,
    # and consequently this action isn't provided to the user!
    #
    # def delete(self):
    #     cal_client = self.calibration_client
    #     resp = cal_client.delete_calibration_constant_version_api(self.id)
    #     Base.cal_debug(MODULE_NAME, DELETE, resp)
    #     return Base.format_response(resp, DELETE, NO_CONTENT, MODULE_NAME)

    def update(self):
        cal_client = self.calibration_client
        resp = cal_client.update_calibration_constant_version_api(
            self.id,
            self.__get_resource()
        )

        Base.cal_debug(MODULE_NAME, UPDATE, resp)
        return Base.format_response(resp, UPDATE, HTTPStatus.OK, MODULE_NAME)

    @staticmethod
    def create_from_dict(cal_client, ccv):
        new_ccv = CalibrationConstantVersion(
            calibration_client=cal_client,
            name=ccv['name'],
            file_name=ccv['file_name'],
            path_to_file=ccv['path_to_file'],
            data_set_name=ccv['data_set_name'],
            calibration_constant_id=ccv['calibration_constant_id'],
            physical_detector_unit_id=ccv['physical_detector_unit_id'],
            flg_deployed=ccv['flg_deployed'],
            flg_good_quality=ccv['flg_good_quality'],
            begin_validity_at=ccv['begin_validity_at'],
            end_validity_at=ccv['end_validity_at'],
            begin_at=ccv['begin_at'],
            start_idx=ccv['start_idx'],
            end_idx=ccv['end_idx'],
            raw_data_location=ccv['raw_data_location'],
            report_id=ccv['report_id'],
            description=ccv['description'])

        resp = new_ccv.create()
        return resp

    @staticmethod
    def get_by_id(cal_client, ccv_id):
        resp = cal_client.get_calibration_constant_version_by_id_api(ccv_id)

        Base.cal_debug(MODULE_NAME, 'get_by_id', resp)
        return Base.format_response(resp, GET, HTTPStatus.OK, MODULE_NAME)

    @staticmethod
    def get_all_by_name(cal_client, name, page=DEF_PAGE, page_size=DEF_PAGE_SIZE):
        resp = cal_client.get_all_calibration_constant_versions_by_name_api(
            name, page, page_size
        )

        Base.cal_debug(MODULE_NAME, 'get_all_by_name', resp)
        return Base.format_response(resp, GET, HTTPStatus.OK, MODULE_NAME)

    @staticmethod
    def get_by_name(cal_client, name):
        resp = CalibrationConstantVersion.get_all_by_name(cal_client, name)

        if resp['success']:
            if resp['data'] == []:
                resp_data = []
            else:
                resp_data = resp['data'][0]

            resp = {'success': resp['success'],
                    'info': resp['info'],
                    'app_info': resp['app_info'],
                    'data': resp_data}

        return resp

    # Returns the CCV (of one CC) which the provided event_at is between
    # the CCV.begin_at and CCV.end_at
    @staticmethod
    def get_by_uk(cal_client,
                  calibration_constant_id, physical_detector_unit_id,
                  event_at, snapshot_at):
        resp = cal_client.get_calibration_constant_version_by_uk_api(
            calibration_constant_id, physical_detector_unit_id,
            event_at, snapshot_at
        )

        Base.cal_debug(MODULE_NAME, 'ccv >> get_by_uk', resp)
        return Base.format_response(resp, GET, HTTPStatus.OK, MODULE_NAME)

    # Returns the CCV (of several CCs) for which the provided event_at is between
    # the CCV.begin_at and CCV.end_at
    @staticmethod
    def get_prior_in_time(cal_client,
                          calibration_constant_ids,
                          physical_detector_unit_id,
                          event_at, snapshot_at):
        resp = cal_client.get_prior_in_time_calibration_constant_version_api(
            calibration_constant_ids, physical_detector_unit_id,
            event_at, snapshot_at
        )

        Base.cal_debug(MODULE_NAME,
                       'ccv >> get_prior_in_time',
                       resp)
        return Base.format_response(resp, GET, HTTPStatus.OK, MODULE_NAME)

    # Returns the CCV (of several CCs) with closest relation between the
    # provided event_at and the CCV.begin_at
    @staticmethod
    def get_closest_by_time(cal_client,
                            calibration_constant_ids,
                            physical_detector_unit_id,
                            event_at, snapshot_at):
        resp = cal_client.get_closest_calibration_constant_version_api(
            calibration_constant_ids, physical_detector_unit_id,
            event_at, snapshot_at
        )

        Base.cal_debug(MODULE_NAME,
                       'ccv >> get_closest_calibration_constant_version_api',
                       resp)
        return Base.format_response(resp, GET, HTTPStatus.OK, MODULE_NAME)

    # Returns the CCV (of several CCs) with closest relation between the
    # provided event_at and the CCV.begin_at BY DETECTOR ID
    @staticmethod
    def get_closest_by_time_by_detector(cal_client,
                                        calibration_constant_id,
                                        detector_id,
                                        karabo_da,
                                        event_at, snapshot_at):
        resp = \
            cal_client.get_closest_calibration_constant_version_by_detector_api(  # noqa
                calibration_constant_id, detector_id, karabo_da,
                event_at, snapshot_at
            )

        Base.cal_debug(
            MODULE_NAME,
            'ccv >> get_closest_calibration_constant_version_by_detector_api',
            resp)
        return Base.format_response(resp, GET, HTTPStatus.OK, MODULE_NAME)

    # Returns the CCVs with closest relation between the
    # provided event_at and the CCV.begin_at by detector identifier
    # and by a limited
    # set of conditions not including the PDUs' UUID but assuming
    # that the conditions have been injected in such a way:
    # This methods assumes a specific situation in the CalCat DB
    # with conditions bound to the PDUs via the UUID parameter
    # (parameter_id = 12)
    @staticmethod
    def get_closest_by_time_by_detector_conditions(cal_client,
                                                   detector_identifier,
                                                   calibration_id,
                                                   condition,
                                                   karabo_da='',
                                                   event_at=None,
                                                   snapshot_at=None):
        resp = cal_client.get_calibration_constant_versions_by_detector_conditions_api(  # noqa
            detector_identifier, calibration_id, condition, karabo_da,
            event_at, snapshot_at
        )

        msg = 'ccv >> get_calibration_constant_versions_by_detector_conditions_api'  # noqa
        Base.cal_debug(MODULE_NAME, msg, resp)
        return Base.format_response(resp, GET, HTTPStatus.OK, MODULE_NAME)

    # To return all constant versions
    @staticmethod
    def get_all_versions(cal_client,
                         calibration_constant_ids, physical_detector_unit_id,
                         event_at, snapshot_at):

        resp = cal_client.get_all_calibration_constant_versions_api(
            calibration_constant_ids, physical_detector_unit_id,
            event_at, snapshot_at
        )

        Base.cal_debug(MODULE_NAME,
                       'ccv >> get_all_calibration_constant_versions_api',
                       resp)
        return Base.format_response(resp, GET, HTTPStatus.OK, MODULE_NAME)

    def __get_resource(self):
        calibration_constant_version = {
            CALIBRATION_CONSTANT_VERSION: {
                'name': self.name,
                'file_name': self.file_name,
                'path_to_file': self.path_to_file,
                'data_set_name': self.data_set_name,
                'calibration_constant_id': self.calibration_constant_id,
                'physical_detector_unit_id': self.physical_detector_unit_id,
                'flg_deployed': self.flg_deployed,
                'flg_good_quality': self.flg_good_quality,
                'begin_validity_at': self.begin_validity_at,
                'end_validity_at': self.end_validity_at,
                'begin_at': self.begin_at,
                'start_idx': self.start_idx,
                'end_idx': self.end_idx,
                'raw_data_location': self.raw_data_location,
                'report_id': self.report_id,
                'description': self.description
            }
        }

        return calibration_constant_version



__author__ = 'Luís Maia <luis.maia@xfel.eu>'

from .calibration import Calibration
from .calibration_constant import CalibrationConstant
from .calibration_constant_version import CalibrationConstantVersion
from .condition import Condition
from .detector import Detector
from .detector_type import DetectorType
from .parameter import Parameter
from .physical_detector_unit import PhysicalDetectorUnit
from .report import Report



"""PhysicalDetectorUnitTest class"""

import unittest

import pytest

from .module_base import ModuleBase
from ..common.config_test import PHYSICAL_DETECTOR_UNIT, RESOURCE_NOT_FOUND
from ..common.generators import generate_unique_name
from ...modules.physical_detector_unit import PhysicalDetectorUnit

MODULE_NAME = PHYSICAL_DETECTOR_UNIT


@pytest.mark.usefixtures('client_cls')
class PhysicalDetectorUnitTest(ModuleBase, unittest.TestCase):
    def setUp(self):
        __unique_name1 = generate_unique_name('PhysicalDev01')
        self.pd_01 = {
            'physical_name': __unique_name1,
            'karabo_da': __unique_name1,
            'virtual_device_name': __unique_name1,
            'uuid': 1001,
            'detector_type_id': -1,
            'detector_id': -1,
            'flg_available': True,
            'description': 'desc 01'
        }

        __unique_name_upd = generate_unique_name('PhysicalDevUpd01')
        self.pd_01_upd = {
            'physical_name': __unique_name1,
            'karabo_da': __unique_name_upd,
            'virtual_device_name': __unique_name_upd,
            'detector_type_id': -1,
            'detector_id': -1,
            'flg_available': False,
            'description': 'desc 01 Updated!!!'
        }

    def test_create_physical_detector_unit(self):
        pd_01 = PhysicalDetectorUnit(
            calibration_client=self.cal_client,
            physical_name=self.pd_01['physical_name'],
            karabo_da=self.pd_01['karabo_da'],
            virtual_device_name=self.pd_01['virtual_device_name'],
            uuid=self.pd_01['uuid'],
            detector_type_id=self.pd_01['detector_type_id'],
            detector_id=self.pd_01['detector_id'],
            flg_available=self.pd_01['flg_available'],
            description=self.pd_01['description'])

        # Create new entry (should succeed)
        result1 = pd_01.create()
        self.assert_create_success(MODULE_NAME, result1, self.pd_01)

        physical_dev_id = result1['data']['id']
        pdu_phy_name = result1['data']['physical_name']

        try:
            # Create duplicated entry (should throw an error)
            result2 = pd_01.create()
            expect_app_info = {'physical_name': ['has already been taken'],
                               'karabo_da': ['has already been taken']}
            self.assert_create_error(MODULE_NAME, result2, expect_app_info)

            # Get entry by name
            res3 = PhysicalDetectorUnit.get_by_name(self.cal_client,
                                                    pdu_phy_name)
            self.assert_find_success(MODULE_NAME, res3, self.pd_01)

            # Get entry by ID
            result4 = PhysicalDetectorUnit.get_by_id(self.cal_client,
                                                     physical_dev_id)
            self.assert_find_success(MODULE_NAME, result4, self.pd_01)

            # Get entry with non-existent ID (should throw an error)
            physical_detector_unit_id = -666
            result5 = PhysicalDetectorUnit.get_by_id(self.cal_client,
                                                     physical_detector_unit_id)
            self.assert_find_error(MODULE_NAME, result5, RESOURCE_NOT_FOUND)

            # Put entry information (update some fields should succeed)
            pd_01.physical_name = self.pd_01_upd['physical_name']
            pd_01.karabo_da = self.pd_01_upd['karabo_da']
            pd_01.virtual_device_name = self.pd_01_upd['virtual_device_name']
            pd_01.flg_available = self.pd_01_upd['flg_available']
            pd_01.description = self.pd_01_upd['description']
            result6 = pd_01.update()
            self.assert_update_success(MODULE_NAME, result6, self.pd_01_upd)

            # Put entry information (update some fields should throw an error)
            pd_01.physical_name = '__THIS_NAME_CANNOT_BE_UPDATED'  # noqa
            pd_01.flg_available = self.pd_01_upd['flg_available']
            pd_01.description = self.pd_01_upd['description']
            result7 = pd_01.update()
            expect_app_info = {
                'physical_name': ['cannot be updated']}
            self.assert_update_error(MODULE_NAME, result7, expect_app_info)

        finally:
            # Delete entry (should succeed)
            # (test purposes only to keep the DB clean)
            result8 = pd_01.delete()

        # Check deletion worked if there isn't already an error to report
        self.assert_delete_success(MODULE_NAME, result8)

        # Delete entry (should throw an error)
        # (test purposes only to keep the DB clean)
        result9 = pd_01.delete()
        self.assert_delete_error(MODULE_NAME, result9, RESOURCE_NOT_FOUND)

    def fields_validation(self, receive, expect):
        assert receive['physical_name'] == expect['physical_name']
        assert receive['karabo_da'] == expect['karabo_da']
        assert receive['virtual_device_name'] == expect['virtual_device_name']
        assert receive['detector_type_id'] == expect['detector_type_id']
        assert receive['detector_id'] == expect['detector_id']
        assert receive['flg_available'] == expect['flg_available']
        assert receive['description'] == expect['description']


if __name__ == '__main__':
    unittest.main()



"""CalibrationConstantVersionApi class"""

import json

from ..common.base import Base
from ..common.config import *


class CalibrationConstantVersionApi(Base):
    def create_calibration_constant_version_api(self, ccv):
        api_url = self.__get_api_url()
        return self.api_post(api_url, data=json.dumps(ccv))

    #
    # Delete CalibrationConstantVersion instances is not allowed by the server,
    # and consequently this action isn't provided to the user!
    #
    # def delete_calibration_constant_version_api(self, ccv_id):
    #     api_url = self.__get_api_url(ccv_id)
    #     return self.api_delete(api_url)

    def update_calibration_constant_version_api(self, ccv_id, ccv):
        api_url = self.__get_api_url(ccv_id)
        return self.api_put(api_url, data=json.dumps(ccv))

    def get_calibration_constant_version_by_id_api(self, ccv_id):
        api_url = self.__get_api_url(ccv_id)
        return self.api_get(api_url, params={})

    def get_all_calibration_constant_versions_by_name_api(self, name,
                                                          page=DEF_PAGE,
                                                          page_size=DEF_PAGE_SIZE):
        api_url = self.__get_api_url()
        return self.api_get(api_url, params={'name': name,
                                             'page': page,
                                             'page_size': page_size})

    def get_calibration_constant_versions_by_detector_conditions_api(
            self,
            detector_identifier,
            calibration_id,
            condition,
            karabo_da='',
            event_at=None,
            snapshot_at=None):
        api_url = self.__get_api_url('get_by_detector_conditions')
        params = {'detector_identifier': detector_identifier,
                  'calibration_id': str(calibration_id),
                  'karabo_da': karabo_da,
                  'event_at': event_at,
                  'snapshot_at': snapshot_at}

        return self.api_get(api_url,
                            params=params,
                            data=json.dumps(condition))

    #
    # Private Methods
    #
    def __get_api_url(self, api_specifics=''):
        model_name = 'calibration_constant_versions/'
        return self.get_api_url(model_name, api_specifics)



"""
This class is a Singleton created to allow us to access the current value
of the Calibration Hash used to test our code from any class and test instance.

That's necessary since we can only successfully retrieve a Calibration
Constant Version if the same was injected to the repository (Database)
"""

import logging
from datetime import datetime, timedelta

from pytz import timezone

__all__ = ['CalibrationTestHash']


class _CalibrationTestHash(object):
    def __init__(self, desired_begin_at=None):
        self.desired_begin_at = desired_begin_at

        self.valid_inject_01 = \
            _CalibrationTestHash._build_valid_test_hash_01(desired_begin_at)
        self.valid_inject_by_det_01 = \
            _CalibrationTestHash._build_valid_test_hash_03(desired_begin_at)
        self.valid_search_01 = \
            _CalibrationTestHash._build_valid_test_hash_02(desired_begin_at)
        self.valid_search_by_det_01 = \
            _CalibrationTestHash._build_valid_test_hash_04(desired_begin_at)

    @staticmethod
    def _build_valid_test_hash_01(desired_begin_at):
        logging.error('>' * 200 + ' _build_valid_test_hash_01 - INJECT')
        ccv_valid_dt = _CalibrationTestHash._gen_valid_ccv_datetime(
            desired_begin_at)

        valid_01_h = {
            'karabo_h': _CalibrationTestHash._build_valid_hash(ccv_valid_dt,
                                                               'inject'),
            'begin_validity_at_exp': ccv_valid_dt['begin_validity_at_exp'],
            'end_validity_at_exp': ccv_valid_dt['end_validity_at_exp'],
            'begin_at_exp': ccv_valid_dt['begin_at_exp']
        }

        return valid_01_h

    @staticmethod
    def _build_valid_test_hash_02(desired_begin_at):
        logging.error('>' * 200 + ' _build_valid_test_hash_02 - SEARCH')

        if desired_begin_at is None:
            desired_begin_at = '2016-10-11T17:57:19.757000'

        ccv_valid_dt = _CalibrationTestHash._gen_valid_ccv_datetime(
            desired_begin_at)

        valid_02_h = {
            'karabo_h': _CalibrationTestHash._build_valid_hash(ccv_valid_dt,
                                                               'search'),
            'begin_validity_at_exp': ccv_valid_dt['begin_validity_at_exp'],
            'end_validity_at_exp': ccv_valid_dt['end_validity_at_exp'],
            'begin_at_exp': ccv_valid_dt['begin_at_exp']
        }

        return valid_02_h

    @staticmethod
    def _build_valid_test_hash_03(desired_begin_at):
        logging.error(
            '>' * 200 + ' _build_valid_test_hash_03 - INJECT BY DETECTOR')
        ccv_valid_dt = _CalibrationTestHash._gen_valid_ccv_datetime(
            desired_begin_at)

        valid_03_h = {
            'karabo_h': _CalibrationTestHash._build_valid_hash(
                ccv_valid_dt, 'inject_by_detector'),
            'begin_validity_at_exp': ccv_valid_dt['begin_validity_at_exp'],
            'end_validity_at_exp': ccv_valid_dt['end_validity_at_exp'],
            'begin_at_exp': ccv_valid_dt['begin_at_exp']
        }

        return valid_03_h

    @staticmethod
    def _build_valid_test_hash_04(desired_begin_at):
        logging.error(
            '>' * 200 + ' _build_valid_test_hash_04 - SEARCH BY DETECTOR')

        if desired_begin_at is None:
            desired_begin_at = '2016-10-11T17:57:19.757000'

        ccv_valid_dt = _CalibrationTestHash._gen_valid_ccv_datetime(
            desired_begin_at)

        valid_04_h = {
            'karabo_h': _CalibrationTestHash._build_valid_hash(
                ccv_valid_dt, 'search_by_detector'),
            'begin_validity_at_exp': ccv_valid_dt['begin_validity_at_exp'],
            'end_validity_at_exp': ccv_valid_dt['end_validity_at_exp'],
            'begin_at_exp': ccv_valid_dt['begin_at_exp']
        }

        return valid_04_h

    @staticmethod
    def _loc(datetime_at, tz_str='Europe/Berlin'):
        tz = timezone(tz_str)

        datetime_at_tz = tz.localize(datetime_at)

        # Datetime should stop in the seconds
        # TODO, Milliseconds should be removed!!!!
        formatted_date_at = datetime_at_tz.isoformat()[:-13] + '.000'
        # formatted_date_at = datetime_at_tz.strftime("%Y-%M-%dT%H:%M:%S")
        # formatted_date_at = '{0}.000'.format(formatted_date_at)

        # And contain the timezone information in numeric values
        formatted_tz = datetime_at_tz.isoformat()[-6:]

        formatted_date_at = '{0}{1}'.format(formatted_date_at, formatted_tz)
        return formatted_date_at

    @staticmethod
    def _gen_valid_ccv_datetime(desired_begin_at):
        # begin_validity_at = '2013-10-16T23:52:41.000+02:00'
        begin_validity_at = '2013-10-17T00:52:41+03:00'
        begin_validity_at_exp = '2013-10-16T23:52:41.000+02:00'

        # NOW == '2015-11-17T16:35:38.006436'
        now = datetime.today()

        # begin_at = 2015-11-17T16:57:19.757+00:00
        # Subtract 1 hour because in Lisbon is one hour less than in Berlin
        insert_begin_at = now + timedelta(hours=-1)
        begin_at = _CalibrationTestHash._loc(insert_begin_at, 'Europe/Lisbon')

        if desired_begin_at is None:
            exp_begin_at = now
        else:
            exp_begin_at = datetime.strptime(desired_begin_at,
                                             '%Y-%m-%dT%H:%M:%S.%f')
            # exp_begin_at = now - timedelta(days=3* 365)

        begin_at_exp = _CalibrationTestHash._loc(exp_begin_at)

        #
        # end_validity_at = 2015-11-17T16:57:20.757+00:00
        # Subtract 1 hour because in Lisbon is one hour less than in Berlin
        insert_end_validity_at = now + timedelta(hours=-2, seconds=1)
        end_validity_at = _CalibrationTestHash._loc(insert_end_validity_at,
                                                    'Atlantic/Azores')

        # end_validity_at_exp = 2015-11-17T16:57:20.757+01:00
        exp_end_validity_at = now + timedelta(seconds=1)
        end_validity_at_exp = _CalibrationTestHash._loc(exp_end_validity_at)

        ccv_test_datetime_h = {
            'begin_validity_at': begin_validity_at,
            'begin_validity_at_exp': begin_validity_at_exp,
            'begin_at': begin_at,
            'begin_at_exp': begin_at_exp,
            'end_validity_at': end_validity_at,
            'end_validity_at_exp': end_validity_at_exp
        }

        return ccv_test_datetime_h

    @staticmethod
    # flg_mode:
    #  * inject -> prepare hash to be injected into DB
    #  * search -> prepare hash to be searched in DB (stable results)
    def _build_valid_hash(ccv_valid_dt, flg_mode):
        cc_name = 'test_inject_cc_unique_name'
        ccv_name = 'test_inject_name_{0}'.format(ccv_valid_dt['begin_at'])

        if flg_mode == 'inject':
            logging.error('>' * 200 + ' INJECT Mode')
            pdu_phy_name = 'PHYSICAL_DETECTOR_UNIT-2_DO_NOT_DELETE'
            # detector_id = -2
            detector_identifier = 'TEST_DET_CI-2'
            pdu_karabo_da = 'TEST_DAQ_DA_02'
            dev_type_name = 'UNIT_TEST_DETECTOR_TYPE-1_DO_NOT_DELETE'
            cal_name = 'CALIBRATION_TEST-1_DO_NOT_DELETE'
            param_1_val = 123.0
            param_2_val = 10.0

        elif flg_mode == 'inject_by_detector':
            logging.error('>' * 200 + ' INJECT Mode')
            pdu_phy_name = ''  # 'PHYSICAL_DETECTOR_UNIT-2_DO_NOT_DELETE'
            # detector_id = -2
            detector_identifier = 'TEST_DET_CI-2'
            pdu_karabo_da = 'TEST_DAQ_DA_02'
            dev_type_name = 'UNIT_TEST_DETECTOR_TYPE-1_DO_NOT_DELETE'
            cal_name = 'CALIBRATION_TEST-1_DO_NOT_DELETE'
            param_1_val = 123.0
            param_2_val = 10.0

        elif flg_mode == 'search':
            logging.error('>' * 200 + ' SEARCH Mode')
            pdu_phy_name = 'PHYSICAL_DETECTOR_UNIT-1_DO_NOT_DELETE'
            # detector_id = -2
            detector_identifier = 'TEST_DET_CI-2'
            pdu_karabo_da = 'TEST_DAQ_DA_01'
            dev_type_name = 'UNIT_TEST_DETECTOR_TYPE-1_DO_NOT_DELETE'
            cal_name = 'CALIBRATION_TEST-2_DO_NOT_DELETE'
            param_1_val = 2.9
            param_2_val = 1.3

        elif flg_mode == 'search_by_detector':
            logging.error('>' * 200 + ' SEARCH Mode')
            pdu_phy_name = ''  # 'PHYSICAL_DETECTOR_UNIT-1_DO_NOT_DELETE'
            # detector_id = -2
            detector_identifier = 'TEST_DET_CI-2'
            pdu_karabo_da = 'TEST_DAQ_DA_01'
            dev_type_name = 'UNIT_TEST_DETECTOR_TYPE-1_DO_NOT_DELETE'
            cal_name = 'CALIBRATION_TEST-2_DO_NOT_DELETE'
            param_1_val = 2.9
            param_2_val = 1.3

        else:
            logging.error('>' * 200 + ' NOT INJECT/SEARCH Mode')
            pdu_phy_name = 'PHYSICAL_DETECTOR_UNIT-3_DO_NOT_DELETE'
            # detector_id = -2
            detector_identifier = 'TEST_DET_CI-2'
            pdu_karabo_da = 'TEST_DAQ_DA_03'
            dev_type_name = 'UNIT_TEST_DETECTOR_TYPE-3_DO_NOT_DELETE'
            cal_name = 'CALIBRATION_TEST-3_DO_NOT_DELETE'
            param_1_val = 300.0
            param_2_val = 600.0

        report_h = {
            # If present, name and file_path must be unique!
            'name': 'this_is_my_1st_report.pdf',
            'file_path': '/gpfs/path/to/report/file/',
            'description': '',
        }

        detector_condition_h = {
            # 'name': '',  # If present, must be unique
            'flg_available': 1,  # [Default == 1 == True]
            'description': '',
            'parameters': [
                {
                    'parameter_name': 'PARAMETER_TEST-1_DO_NOT_DELETE',
                    'value': param_1_val,
                    'flg_logarithmic': 0,  # [Default == 0 == False]
                    'lower_deviation_value': 2,
                    'upper_deviation_value': 1,
                    'flg_available': 1,  # [Default == 0 == False]
                    'description': ''
                },
                {
                    'parameter_name': 'PARAMETER_TEST-2_DO_NOT_DELETE',
                    'value': param_2_val,
                    'flg_logarithmic': 0,  # [Default == 0 == False]
                    'lower_deviation_value': 0.1,
                    'upper_deviation_value': 0.05,
                    'flg_available': 1,  # [Default == 0 == False]
                    'description': ''
                }
            ]
        }

        cc_h = {
            'name': cc_name,  # If present, must be unique
            'detector_type_name': dev_type_name,
            'calibration_name': cal_name,
            'flg_available': 1,  # [Default == 1 == True]
            'flg_auto_approve': 1,
            'description': '',
        }

        path_to_file = 'xfel/cal/{0}/{1}/'.format(dev_type_name, pdu_phy_name)
        ccv_h = {
            'name': ccv_name,  # If present, must be unique

            # If pdu_physical_name is present,
            # detector_identifier and pdu_karabo_da are ignored
            'pdu_physical_name': pdu_phy_name,
            'detector_identifier': detector_identifier,
            'pdu_karabo_da': pdu_karabo_da,
            # end comment

            'path_to_file': path_to_file,
            'file_name': 'test_01',
            'flg_good_quality': 1,
            'begin_validity_at': ccv_valid_dt['begin_validity_at'],
            'end_validity_at': ccv_valid_dt['end_validity_at'],
            'begin_at': ccv_valid_dt['begin_at'],
            'raw_data_location': '/somewhere',
            'description': '',
            'start_idx': 0,
            'end_idx': 1,
            'data_set_name': cc_h['calibration_name']
        }

        valid_dict = {
            'report': report_h,
            'detector_condition': detector_condition_h,
            'calibration_constant': cc_h,
            'calibration_constant_version': ccv_h
        }

        return valid_dict


def init(desired_begin_at):
    global cal_test_hash
    cal_test_hash = _CalibrationTestHash(desired_begin_at)


def CalibrationTestHash(desired_begin_at):
    init(desired_begin_at)
    return cal_test_hash



"""Tests for utilities"""

from ...common.util import Util


def test_int_val_to_api_bool():
    assert Util.val_to_api_bool(-1) == 'false'
    assert Util.val_to_api_bool(0) == 'false'
    assert Util.val_to_api_bool(1) == 'true'
    assert Util.val_to_api_bool(2) == 'false'
    assert Util.val_to_api_bool(3) == 'false'
    assert Util.val_to_api_bool(99) == 'false'


def test_bool_val_to_api_bool():
    assert Util.val_to_api_bool(False) == 'false'
    assert Util.val_to_api_bool(True) == 'true'


def test_error_val_to_api_bool():
    # String
    assert Util.val_to_api_bool(
        'False') == "Error: Value False is of type <class 'str'>"
    assert Util.val_to_api_bool(
        'True') == "Error: Value True is of type <class 'str'>"

    # Double
    float_error_msg = "Error: Value 2.23232e-07 is of type <class 'float'>"
    assert Util.val_to_api_bool(0.000000223232) == float_error_msg


def test_get_opt_hash_val():
    h01 = {
        'test_01': 'value_01',
        'test_02': 'value_02',
        # 'test_03': '',
        'test_04': ''
    }
    val01 = Util.get_opt_hash_val(h01, 'test_01', "I'm the default value!")
    val01_without_default = Util.get_opt_hash_val(h01, 'test_01')
    assert val01 == 'value_01'
    assert val01_without_default == 'value_01'

    val02 = Util.get_opt_hash_val(h01, 'test_02', "I'm the default value!")
    val02_without_default = Util.get_opt_hash_val(h01, 'test_02')
    assert val02 == 'value_02'
    assert val02_without_default == 'value_02'

    val03 = Util.get_opt_hash_val(h01, 'test_03', "I'm the default value!")
    val03_without_default = Util.get_opt_hash_val(h01, 'test_03')
    assert val03 == "I'm the default value!"
    assert val03_without_default == ''

    val04 = Util.get_opt_hash_val(h01, 'test_04', "I'm the default value!")
    val04_without_default = Util.get_opt_hash_val(h01, 'test_04')
    assert val04 == ''
    assert val04_without_default == ''



"""Tests for Base class"""

from ...common.base import Base


def test_load_json_from_str():
    json_01 = Base.load_json_from_str('')
    assert json_01 == {}

    json_02 = Base.load_json_from_str('{"hello": "world"}')
    assert json_02 == {'hello': 'world'}


def test_response_success():
    # CREATE
    res = Base.response_success('MOD_01', 'CREATE', '{pagination hash}', '{content hash}')
    expected_res = {'success': True, 'info': 'MOD_01 created successfully',
                    'app_info': {}, 'data': '{content hash}',
                    'pagination': '{pagination hash}'}
    assert res == expected_res

    # UPDATE
    res = Base.response_success('MOD_02', 'UPDATE',
                                '{pagination hash}', '{content hash}')
    expected_res = {'success': True, 'info': 'MOD_02 updated successfully',
                    'app_info': {}, 'data': '{content hash}',
                    'pagination': '{pagination hash}'}
    assert res == expected_res

    # GET
    res = Base.response_success('MOD_03', 'GET',
                                '{pagination hash}', '{content hash}')
    expected_res = {'success': True, 'info': 'Got MOD_03 successfully',
                    'app_info': {}, 'data': '{content hash}',
                    'pagination': '{pagination hash}'}
    assert res == expected_res

    # DELETE
    res = Base.response_success('MOD_04', 'DELETE', {}, '{content hash}')
    expected_res = {'success': True, 'info': 'MOD_04 deleted successfully',
                    'app_info': {}, 'data': '{content hash}',
                    'pagination': {}}
    assert res == expected_res

    # SET
    res = Base.response_success('MOD_05', 'SET', {}, '{content hash}')
    expected_res = {'success': True, 'info': 'MOD_05 set successfully',
                    'app_info': {}, 'data': '{content hash}',
                    'pagination': {}}
    assert res == expected_res

    # OTHER_ACTION
    res = Base.response_success('MOD_06', 'OTHER_ACTION', {}, '{content hash}')
    expected_res = {'success': False, 'info': 'ACTION is not correct!',
                    'app_info': '{content hash}', 'data': {},
                    'pagination': {}}
    assert res == expected_res


def test_response_error():
    # CREATE
    res = Base.response_error('MOD_01', 'CREATE', 'Error 01')
    expected_res = {'success': False, 'info': 'Error creating MOD_01',
                    'app_info': 'Error 01', 'data': {},
                    'pagination': {}}
    assert res == expected_res

    # UPDATE
    res = Base.response_error('MOD_02', 'UPDATE', 'Error 02')
    expected_res = {'success': False, 'info': 'Error updating MOD_02',
                    'app_info': 'Error 02', 'data': {},
                    'pagination': {}}
    assert res == expected_res

    # GET
    res = Base.response_error('MOD_03', 'GET', 'Error 03')
    expected_res = {'success': False, 'info': 'MOD_03 not found!',
                    'app_info': 'Error 03', 'data': {},
                    'pagination': {}}
    assert res == expected_res

    # DELETE
    res = Base.response_error('MOD_04', 'DELETE', 'Error 04')
    expected_res = {'success': False, 'info': 'Error deleting MOD_04',
                    'app_info': 'Error 04', 'data': {},
                    'pagination': {}}
    assert res == expected_res

    # SET
    res = Base.response_error('MOD_05', 'SET', 'Error 05')
    expected_res = {'success': False, 'info': 'Error setting MOD_05',
                    'app_info': 'Error 05', 'data': {},
                    'pagination': {}}
    assert res == expected_res

    # OTHER_ACTION
    res = Base.response_error('MOD_06', 'OTHER_ACTION', 'Error 06')
    expected_res = {'success': False, 'info': 'ACTION is not correct!',
                    'app_info': 'Error 06', 'data': {},
                    'pagination': {}}
    assert res == expected_res



"""Base Class with helper methods common to all modules"""

import json
import logging
from http import HTTPStatus

import requests
from oauth2_xfel_client.oauth2_client_backend import Oauth2ClientBackend

from ..common.config import CREATE, DELETE, UPDATE, GET, SET, \
    DEF_MAX_RETRIES, DEF_TIMEOUT, DEF_SSL_VERIFY


def default_headers(user_email):
    return {
        'content-type': 'application/json',
        'Accept': 'application/json; version=2',
        'X-User-Email': user_email,
    }


class Base:
    def __init__(self, *,
                 base_api_url, use_oauth2=True,
                 client_id='', client_secret='',
                 token_url='', refresh_url='', auth_url='', scope='',
                 user_email='',
                 session_token=None,
                 max_retries=DEF_MAX_RETRIES,
                 timeout=DEF_TIMEOUT,
                 ssl_verify=DEF_SSL_VERIFY):

        if use_oauth2:
            self.oauth_client = Oauth2ClientBackend(
                client_id=client_id,
                client_secret=client_secret,
                scope=scope,
                token_url=token_url,
                refresh_url=refresh_url,
                auth_url=auth_url,
                session_token=session_token,
                max_retries=max_retries,
                timeout=timeout,
                ssl_verify=ssl_verify
            )
            self.session = self.oauth_client.session

        else:
            # Oauth disabled - used with base_api_url pointing to an
            # xfel-oauth-proxy instance
            self.oauth_client = None
            self.session = requests.Session()

        self.headers = default_headers(user_email)
        # Ensure the base URL has a trailing slash
        self.base_api_url = base_api_url.rstrip('/') + '/'

    @staticmethod
    def load_json_from_str(hash_str):
        if hash_str == '' or hash_str == '[]':
            return {}
        else:
            return json.loads(hash_str)

    @staticmethod
    def load_json_from_content(response):
        return Base.load_json_from_str(response.content.decode('utf8'))

    @staticmethod
    def load_json_from_headers(response):
        headers = {}

        try:
            if response.headers is None:
                return headers
        except Exception as exp:
            return headers

        headers['Date'] = response.headers.get('Date')
        headers['X-Total-Pages'] = response.headers.get('X-Total-Pages')
        headers['X-Count-Per-Page'] = response.headers.get('X-Count-Per-Page')
        headers['X-Current-Page'] = response.headers.get('X-Current-Page')
        headers['X-Total-Count'] = response.headers.get('X-Total-Count')

        return headers

    @staticmethod
    def format_response(response, action, success_code, module_name):
        r_content = Base.load_json_from_content(response)
        r_pagination = Base.load_json_from_headers(response)

        # print('-' * 100)
        # print('r_content: ' + str(r_content))
        # print('action: ' + str(action))
        # print('success_code: ' + str(success_code))
        # print('module_name: ' + str(module_name))

        # Standard situation in case of success
        if response.status_code == success_code and r_content:
            res = Base.response_success(module_name, action,
                                        r_pagination, r_content)

        # Successful set operation on Reports may return 200 or 201
        elif action == SET and response.status_code in [HTTPStatus.OK, HTTPStatus.CREATED] and r_content:
            res = Base.response_success(module_name, action,
                                        r_pagination, r_content)

        # In case there is nothing to report (e.g. delete operation success)
        elif response.status_code == HTTPStatus.NO_CONTENT and not r_content:
            res = Base.response_success(module_name, action,
                                        r_pagination, r_content)

        # This else if covers search return data (e.g. find_by_name)
        elif response.status_code == success_code and r_content == []:
            res = Base.response_success(module_name, action,
                                        r_pagination, r_content)

        # Protects from having a success response, but an empty hash was sent
        elif response.status_code == success_code and r_content == {}:
            res = Base.response_error(module_name, action, r_content)

        # In any other case an Error is reported
        else:
            if 'info' in r_content:
                app_info = r_content['info']
            else:
                app_info = '{}: expected {}, got {}'.format(
                    "HTTP response unexpected status code",
                    success_code, response.status_code)
            res = Base.response_error(module_name, action, app_info)

        return res

    @staticmethod
    # TODO, Make it validate and return key, value criteria
    def unique_key_format_result(res, module_name, unique_id=None):
        if len(res['data']) > 0:
            data = res['data'][0]
            info = res['info']
        else:
            data = {}

            if unique_id:
                info = '{0} "{1}" could not be found!'.format(module_name,
                                                              unique_id)
            else:
                info_msg = 'No {0} could be found respecting the specified ' \
                           'search criteria'
                info = info_msg.format(module_name)

        res = {'success': res['success'],
               'info': info,
               'app_info': res['app_info'],
               'pagination': Base.load_json_from_headers(res),
               'data': data}
        return res

    @staticmethod
    def response_success(module_name, action, r_pagination, r_content):
        if action == CREATE:
            msg = '{0} created successfully'.format(module_name)
        elif action == UPDATE:
            msg = '{0} updated successfully'.format(module_name)
        elif action == GET:
            msg = 'Got {0} successfully'.format(module_name)
        elif action == DELETE:
            msg = '{0} deleted successfully'.format(module_name)
        elif action == SET:
            msg = '{0} set successfully'.format(module_name)
        else:
            return Base.response_error(module_name, action, r_content)

        res = {'success': True,
               'info': msg,
               'app_info': {},
               'pagination': r_pagination,
               'data': r_content}
        logging.debug('response_success => {0}'.format(res))
        return res

    @staticmethod
    def response_error(module_name, action, app_info):
        if action == CREATE:
            msg = 'Error creating {0}'.format(module_name)
        elif action == UPDATE:
            msg = 'Error updating {0}'.format(module_name)
        elif action == GET:
            msg = '{0} not found!'.format(module_name)
        elif action == DELETE:
            msg = 'Error deleting {0}'.format(module_name)
        elif action == SET:
            msg = 'Error setting {0}'.format(module_name)
        else:
            msg = 'ACTION is not correct!'

        res = {'success': False,
               'info': msg,
               'app_info': app_info,
               'pagination': {},
               'data': {}}
        logging.debug('response_error => {0}'.format(res))
        return res

    @staticmethod
    def cal_debug(c_name, m_name, r_content):
        msg = '*** {0}.{1} (content) => {2}'.format(c_name, m_name, r_content)
        logging.debug(msg)

    # Method that generates the RESTful API URL
    def get_api_url(self, model_name, api_specifics=''):
        complete_api_specifics = '{0}{1}'.format(model_name, api_specifics)
        return '{0}{1}'.format(self.base_api_url, complete_api_specifics)

    def api_get(self, api_url, **kwargs):
        """Sends a GET request. Returns :class:`Response` object.
        :param api_url: URL for the new :class:`Request` object.
        :param kwargs: Optional arguments that ``request`` takes.
        """
        kwargs.setdefault('allow_redirects', True)

        return self.session.get(api_url, headers=self.headers, **kwargs)

    def api_post(self, api_url, **kwargs):
        """Sends a POST request. Returns :class:`Response` object.
        :param api_url: URL for the new :class:`Request` object.
        :param kwargs: Optional arguments that ``request`` takes.
        """
        kwargs.setdefault('allow_redirects', True)

        return self.session.post(api_url, headers=self.headers, **kwargs)

    def api_put(self, api_url, **kwargs):
        """Sends a PUT request. Returns :class:`Response` object.
        :param api_url: URL for the new :class:`Request` object.
        :param kwargs: Optional arguments that ``request`` takes.
        """
        kwargs.setdefault('allow_redirects', True)

        return self.session.put(api_url, headers=self.headers, **kwargs)

    def api_delete(self, api_url, **kwargs):
        """Sends a DELETE request. Returns :class:`Response` object.
        :param api_url: URL for the new :class:`Request` object.
        :param kwargs: Optional arguments that ``request`` takes.
        """
        kwargs.setdefault('allow_redirects', True)

        return self.session.delete(api_url, headers=self.headers, **kwargs)



"""Util Class with helper methods"""
import datetime


class Util(object):
    @staticmethod
    def val_to_api_bool(value_in):
        if not isinstance(value_in, int):  # bool inherits from int
            # FIXME: raise!
            return 'Error: Value {} is of type {}'.format(
                value_in, type(value_in)
            )

        # FIXME: values other than 0 or 1 probably shouldn't mean false
        return 'true' if (value_in == 1) else 'false'

    @staticmethod
    def get_opt_hash_val(h, element_name, def_val=''):
        if element_name in h:
            return h[element_name]
        else:
            return def_val

    @staticmethod
    def datetime_converter(o):
        if isinstance(o, datetime.datetime):
            return str(o)



"""CalibrationConstantApi class"""

import json

from ..common.base import Base
from ..common.config import *


class CalibrationConstantApi(Base):
    def create_calibration_constant_api(self, cal_constant):
        api_url = self.__get_api_url()
        return self.api_post(api_url, data=json.dumps(cal_constant))

    def delete_calibration_constant_api(self, cal_constant_id):
        api_url = self.__get_api_url(cal_constant_id)
        return self.api_delete(api_url)

    def update_calibration_constant_api(self, cal_constant_id, cal_constant):
        api_url = self.__get_api_url(cal_constant_id)
        return self.api_put(api_url, data=json.dumps(cal_constant))

    def get_calibration_constant_by_id_api(self, cal_constant_id):
        api_url = self.__get_api_url(cal_constant_id)
        return self.api_get(api_url, params={})

    def get_all_calibration_constants_by_name_api(self, name,
                                                  page=DEF_PAGE,
                                                  page_size=DEF_PAGE_SIZE):
        api_url = self.__get_api_url()
        return self.api_get(api_url, params={'name': name,
                                             'page': page,
                                             'page_size': page_size})

    #
    def get_calibration_constant_version_by_uk_api(self,
                                                   calibration_constant_id,
                                                   physical_detector_unit_id,
                                                   event_at,
                                                   snapshot_at):
        api_action_name = '/get_version'
        api_relative_url = '{0}{1}'.format(calibration_constant_id,
                                           api_action_name)
        api_url = self.__get_api_url(api_relative_url)

        params = {'physical_detector_unit_id': str(physical_detector_unit_id),
                  'event_at': event_at,
                  'snapshot_at': snapshot_at}

        return self.api_get(api_url, params=params)

    #
    def get_prior_in_time_calibration_constant_version_api(self,
                                                           calibration_constant_ids,
                                                           physical_detector_unit_id,
                                                           event_at,
                                                           snapshot_at):
        api_action_name = '/get_prior_in_time_version'
        api_relative_url = '{0}{1}'.format(calibration_constant_ids[0],
                                           api_action_name)
        api_url = self.__get_api_url(api_relative_url)

        params = {'calibration_constant_ids': str(calibration_constant_ids),
                  'physical_detector_unit_id': str(physical_detector_unit_id),
                  'event_at': event_at,
                  'snapshot_at': snapshot_at}

        return self.api_get(api_url, params=params)

    #
    def get_closest_calibration_constant_version_api(self,
                                                     calibration_constant_ids,
                                                     physical_detector_unit_id,
                                                     event_at,
                                                     snapshot_at):
        api_action_name = '/get_closest_version'
        api_relative_url = '{0}{1}'.format(calibration_constant_ids[0],
                                           api_action_name)
        api_url = self.__get_api_url(api_relative_url)

        params = {'calibration_constant_ids': str(calibration_constant_ids),
                  'physical_detector_unit_id': str(physical_detector_unit_id),
                  'event_at': event_at,
                  'snapshot_at': snapshot_at}

        return self.api_get(api_url, params=params)

    #
    def get_closest_calibration_constant_version_by_detector_api(
            self, calibration_constant_ids, detector_id, karabo_da,
            event_at, snapshot_at):
        api_action_name = '/get_closest_version_by_detector'
        api_relative_url = '{0}{1}'.format(calibration_constant_ids[0],
                                           api_action_name)
        api_url = self.__get_api_url(api_relative_url)

        params = {'calibration_constant_ids': str(calibration_constant_ids),
                  'detector_id': str(detector_id),
                  'karabo_da': str(karabo_da),
                  'event_at': event_at,
                  'snapshot_at': snapshot_at}

        return self.api_get(api_url, params=params)

    #
    def get_all_calibration_constant_versions_api(self,
                                                  calibration_constant_ids,
                                                  physical_detector_unit_id,
                                                  event_at,
                                                  snapshot_at):
        api_action_name = '/get_all_versions'
        api_relative_url = '{0}{1}'.format(calibration_constant_ids[0],
                                           api_action_name)
        api_url = self.__get_api_url(api_relative_url)

        params = {'calibration_constant_ids': str(calibration_constant_ids),
                  'physical_detector_unit_id': str(physical_detector_unit_id),
                  'event_at': event_at,
                  'snapshot_at': snapshot_at}

        return self.api_get(api_url, params=params)

    #
    # Private Methods
    #
    def __get_api_url(self, api_specifics=''):
        model_name = 'calibration_constants/'
        return self.get_api_url(model_name, api_specifics)



"""PhysicalDetectorUnit module class"""

from http import HTTPStatus

from ..common.base import Base
from ..common.config import *

MODULE_NAME = PHYSICAL_DETECTOR_UNIT


class PhysicalDetectorUnit:
    def __init__(self, calibration_client,
                 physical_name, karabo_da, virtual_device_name,
                 uuid, detector_type_id, detector_id,
                 flg_available, description=''):
        self.calibration_client = calibration_client
        self.id = None
        self.physical_name = physical_name
        self.karabo_da = karabo_da
        self.virtual_device_name = virtual_device_name
        self.uuid = uuid
        self.detector_type_id = detector_type_id
        self.detector_id = detector_id
        self.flg_available = flg_available
        self.description = description

    def create(self):
        cal_client = self.calibration_client
        response = cal_client.create_physical_detector_unit_api(
            self.__get_resource())

        Base.cal_debug(MODULE_NAME, CREATE, response)
        res = Base.format_response(
            response, CREATE, HTTPStatus.CREATED, MODULE_NAME
        )

        if res['success']:
            self.id = res['data']['id']

        return res

    def delete(self):
        cal_client = self.calibration_client
        response = cal_client.delete_physical_detector_unit_api(self.id)

        Base.cal_debug(MODULE_NAME, DELETE, response)
        return Base.format_response(
            response, DELETE, HTTPStatus.NO_CONTENT, MODULE_NAME
        )

    def update(self):
        cal_client = self.calibration_client
        resp = cal_client.update_physical_detector_unit_api(
            self.id, self.__get_resource())

        Base.cal_debug(MODULE_NAME, UPDATE, resp)
        return Base.format_response(
            resp, UPDATE, HTTPStatus.OK, MODULE_NAME
        )

    @staticmethod
    def get_by_id(cal_client, physical_detector_unit_id):
        response = cal_client.get_physical_detector_unit_by_id_api(
            physical_detector_unit_id)

        Base.cal_debug(MODULE_NAME, 'get_by_id', response)
        return Base.format_response(response, GET, HTTPStatus.OK, MODULE_NAME)

    @staticmethod
    def get_all_by_name(cal_client, physical_name,
                        page=DEF_PAGE, page_size=DEF_PAGE_SIZE):

        resp = cal_client.get_all_physical_detector_units_by_physical_name_api(
            physical_name, page, page_size)

        Base.cal_debug(MODULE_NAME, 'get_all_by_name', resp)
        return Base.format_response(resp, GET, HTTPStatus.OK, MODULE_NAME)

    @staticmethod
    def get_all_by_detector_id(cal_client, det_id,
                               page=DEF_PAGE, page_size=DEF_PAGE_SIZE):
        resp = cal_client.get_all_physical_detector_units_by_detector_id_api(
            det_id, page, page_size)

        Base.cal_debug(MODULE_NAME, 'get_all_by_detector_id', resp)
        return Base.format_response(resp, GET, HTTPStatus.OK, MODULE_NAME)

    @staticmethod
    def get_all_by_detector_and_karabo_da(cal_client, det_id, krb_da,
                                          page=DEF_PAGE, page_size=DEF_PAGE_SIZE):

        resp = cal_client.get_all_physical_detector_units_by_det_and_krbda_api(
            det_id,
            krb_da,
            page,
            page_size)

        Base.cal_debug(MODULE_NAME, 'get_all_by_detector_and_karabo_da', resp)
        return Base.format_response(resp, GET, HTTPStatus.OK, MODULE_NAME)

    @staticmethod
    def get_all_by_detector(cal_client, detector_id, snapshot_at):
        resp = cal_client.get_all_by_detector_api(detector_id, snapshot_at)

        Base.cal_debug(MODULE_NAME, 'ccv >> get_all_by_detector', resp)
        return Base.format_response(resp, GET, HTTPStatus.OK, MODULE_NAME)

    @staticmethod
    def get_by_name(cal_client, physical_name):
        res = PhysicalDetectorUnit.get_all_by_name(cal_client, physical_name)

        if res['success']:
            if res['data'] == []:
                resp_data = []
            else:
                resp_data = res['data'][0]

            res = {'success': res['success'],
                   'info': res['info'],
                   'app_info': res['app_info'],
                   'data': resp_data}

        return res

    def __get_resource(self):
        physical_detector_unit = {
            PHYSICAL_DETECTOR_UNIT: {
                'physical_name': self.physical_name,
                'karabo_da': self.karabo_da,
                'virtual_device_name': self.virtual_device_name,
                'uuid': self.uuid,
                'detector_type_id': self.detector_type_id,
                'detector_id': self.detector_id,
                'flg_available': self.flg_available,
                'description': self.description
            }
        }

        return physical_detector_unit



"""ReportApiTest class"""

import unittest
from http import HTTPStatus

import pytest

from .api_base import ApiBase
from ..common.config_test import *


@pytest.mark.usefixtures('client_cls')
class ReportApiTest(ApiBase, unittest.TestCase):
    def test_create_report_api(self):
        __unique_name = 'ReportApi-Dynamic-TO_DELETED'
        report = {
            'report': {
                'name': __unique_name,
                'file_path': __unique_name,
                'flg_available': True,
                'description': 'desc 01'
            }
        }

        expect = report['report']

        # Create new entry (should succeed)
        # received = self.__create_entry_api(report, expect)

        created_entry_resp = self.cal_client.create_report_api(report)

        created_entry_received = self.get_and_validate_create_entry(
            created_entry_resp)

        report_id = created_entry_received['id']
        repo_name = created_entry_received['name']
        repo_file_path = created_entry_received['file_path']

        try:
            # Check that expected creation result was received
            self.fields_validation(created_entry_received, expect)

            # Create duplicated entry (should throw an error)
            self.__create_error_entry_uk_api(report)

            # Set duplicated entry
            # (should NOT throw an error and return the report from the DB)
            self.__set_failsafe_duplicate_entry_uk_api(report, expect)

            # Get entry by name
            self.__get_all_entries_by_name_api(repo_name, expect)

            # Get entry by identifier
            self.__get_all_entries_by_name_and_path_api(repo_name,
                                                        repo_file_path,
                                                        expect)

            # Get entry by ID
            self.__get_entry_by_id_api(report_id, expect)

            # Put entry information (update some fields should succeed)
            self.__update_entry_api(report_id, expect)

        finally:
            # Delete entry (should succeed)
            # (test purposes only to keep the DB clean)
            del_resp = self.cal_client.delete_report_api(report_id)

        # Validate the response from deletion if nothing already error-ed
        self.get_and_validate_delete_entry_by_id(del_resp)

    def fields_validation(self, receive, expect):
        assert receive['name'] == expect['name']
        assert receive['file_path'] == expect['file_path']
        assert receive['flg_available'] == expect['flg_available']
        assert receive['description'] == expect['description']

    #
    # Internal private APIs methods
    #
    def __create_entry_api(self, entry_info, expect):
        response = self.cal_client.create_report_api(entry_info)

        receive = self.get_and_validate_create_entry(response)
        self.fields_validation(receive, expect)
        return receive

    def __create_error_entry_uk_api(self, entry_info):
        response = self.cal_client.create_report_api(entry_info)
        receive = self.load_response_content(response)
        expect = {'info': {'name': ['has already been taken'],
                           'file_path': ['has already been taken']}}
        assert receive == expect
        assert response.status_code == HTTPStatus.UNPROCESSABLE_ENTITY

    def __set_failsafe_duplicate_entry_uk_api(self, entry_info, expect):
        response = self.cal_client.set_report_api(entry_info)
        receive = self.get_and_validate_set_entry(response)
        self.fields_validation(receive, expect)

    def __update_entry_api(self, entry_id, expect):
        unique_name_upd = 'ReportApi-Dynamic_UPD-TO_DELETED'
        report_upd = {
            'report': {
                'name': unique_name_upd,
                'file_path': unique_name_upd,
                'flg_available': False,
                'description': 'desc 01 updated!!!'
            }
        }

        response = self.cal_client.update_report_api(entry_id,
                                                     report_upd)
        receive = self.load_response_content(response)

        expect_upd = report_upd['report']

        self.fields_validation(receive, expect_upd)
        assert response.status_code == HTTPStatus.OK

        assert expect['name'] != expect_upd['name']
        assert receive['file_path'] != expect['file_path']
        assert expect['flg_available'] != expect_upd['flg_available']
        assert expect['description'] != expect_upd['description']

    def __get_all_entries_by_name_api(self, name, expect,
                                      page=DEF_PAGE, page_size=DEF_PAGE_SIZE):
        response = self.cal_client.get_all_reports_by_name_api(name, page, page_size)
        receive = self.get_and_validate_all_entries_by_name(response)
        self.fields_validation(receive, expect)

    def __get_all_entries_by_name_and_path_api(self, name, file_path, expect):
        resp = self.cal_client.get_all_reports_by_name_and_file_path_api(
            name, file_path)
        receive = self.get_and_validate_all_entries_by_name(resp)
        self.fields_validation(receive, expect)

    def __get_entry_by_id_api(self, entry_id, expect):
        response = self.cal_client.get_report_by_id_api(entry_id)
        receive = self.get_and_validate_entry_by_id(response)
        self.fields_validation(receive, expect)


if __name__ == '__main__':
    unittest.main()



"""CalibrationConstantApiTest class"""

import unittest
from http import HTTPStatus

import pytest

from .api_base import ApiBase
from ..common.generators import generate_unique_name
from ..common.config_test import *


@pytest.mark.usefixtures('client_cls')
class CalibrationConstantApiTest(ApiBase, unittest.TestCase):
    db_cal_const1 = {
        'calibration_constant': {
            'name': 'CALIBRATION_CONSTANT_TEST-1_DO_NOT_DELETE',
            'calibration_id': -1,
            'detector_type_id': -1,
            'condition_id': -1,
            'description': 'Created automatically via seed:seed_tests'
        }
    }
    db_cal_const2 = {
        'calibration_constant': {
            'name': 'CALIBRATION_CONSTANT_TEST-2_DO_NOT_DELETE',
            'calibration_id': -1,
            'detector_type_id': -1,
            'condition_id': -2,
            'description': 'Created automatically via seed:seed_tests'
        }
    }

    def test_create_calibration_constant_api(self):
        __unique_name = generate_unique_name('CConstantApi')
        calibration_const = {
            'calibration_constant': {
                'name': __unique_name,
                'calibration_id': -2,
                'detector_type_id': -1,
                'condition_id': -1,
                'flg_auto_approve': True,
                'flg_available': True,
                'description': 'desc 01'
            }
        }

        expect = calibration_const['calibration_constant']

        expect_db_all = [self.db_cal_const1['calibration_constant'],
                         self.db_cal_const2['calibration_constant']]

        # Create new entry (should succeed)
        create_entry_resp = self.cal_client.create_calibration_constant_api(
            calibration_const)

        create_entry_received = self.get_and_validate_create_entry(
            create_entry_resp)

        cal_constant_id = create_entry_received['id']
        cal_constant_name = create_entry_received['name']

        try:
            # Check that expected creation result was received
            self.fields_validation(create_entry_received, expect)

            # Create duplicated entry (should throw an error)
            self.__create_error_entry_uk_api(calibration_const)

            # Get entry by all calibration constants (should succeed)
            condition_ids = [
                self.db_cal_const1['calibration_constant']['condition_id'],
                self.db_cal_const2['calibration_constant']['condition_id']]

            self.__get_all_calibration_constants_by_conditions_api(
                self.db_cal_const1['calibration_constant']['calibration_id'],
                self.db_cal_const1['calibration_constant']['detector_type_id'],
                condition_ids,
                expect_db_all)

            # Get entry by it's Unique Key (UK) (should succeed)
            self.__get_entry_by_uk_api(calibration_const, expect)

            # Get entry by it's Unique Key (UK) (should not_found)
            self.__get_entry_by_uk_api_not_found_api()

            # Get entry by name
            self.__get_all_entries_by_name_api(cal_constant_name, expect)

            # Get entry by ID
            self.__get_entry_by_id_api(cal_constant_id, expect)

            # Put entry information (update some fields should succeed)
            self.__update_entry_api(cal_constant_id, expect)

        finally:
            # Delete entry (should succeed)
            # (test purposes only to keep the DB clean)
            del_resp = self.cal_client.delete_calibration_constant_api(
                cal_constant_id
            )

        # Validate the response from deletion if nothing already error-ed
        self.get_and_validate_delete_entry_by_id(del_resp)

    def fields_validation(self, receive, expect):
        assert receive['name'] == expect['name']
        assert receive['detector_type_id'] == expect['detector_type_id']
        assert receive['calibration_id'] == expect['calibration_id']
        assert receive['condition_id'] == expect['condition_id']
        assert receive['flg_auto_approve'] == expect['flg_auto_approve']
        assert receive['flg_available'] == expect['flg_available']
        assert receive['description'] == expect['description']

    def __create_error_entry_uk_api(self, entry_info):
        response = self.cal_client.create_calibration_constant_api(
            entry_info)
        receive = self.load_response_content(response)

        expect = {
            'info': {
                'name': ['has already been taken'],
                'detector_type': ['has already been taken'],
                'calibration': ['has already been taken'],
                'condition': ['has already been taken']
            }
        }

        assert receive == expect
        assert response.status_code == HTTPStatus.UNPROCESSABLE_ENTITY

    def __update_entry_api(self, entry_id, expect):
        unique_name = generate_unique_name('CConstantApiUpd')
        calibration_const_upd = {
            'calibration_constant': {
                'name': unique_name,
                # 'calibration_id': '-2',
                # 'detector_type_id': '-1',
                # 'condition_id': '-1',
                'flg_auto_approve': False,
                'flg_available': False,
                'description': 'desc 01 updated!'
            }
        }

        resp = self.cal_client.update_calibration_constant_api(
            entry_id,
            calibration_const_upd
        )

        receive = self.load_response_content(resp)

        # Add parameters not send to the update API
        calibration_const_upd['calibration_constant']['calibration_id'] = -2
        calibration_const_upd['calibration_constant']['detector_type_id'] = -1
        calibration_const_upd['calibration_constant']['condition_id'] = -1
        expect_upd = calibration_const_upd['calibration_constant']

        self.fields_validation(receive, expect_upd)
        assert resp.status_code == HTTPStatus.OK

        assert expect['name'] != expect_upd['name']
        assert expect['flg_auto_approve'] != expect_upd['flg_auto_approve']
        assert expect['flg_available'] != expect_upd['flg_available']
        assert expect['description'] != expect_upd['description']

    def __get_all_entries_by_name_api(self, name, expect,
                                      page=DEF_PAGE, page_size=DEF_PAGE_SIZE):
        resp = self.cal_client.get_all_calibration_constants_by_name_api(
            name, page, page_size
        )

        receive = self.get_and_validate_all_entries_by_name(resp)
        self.fields_validation(receive, expect)

    def __get_entry_by_id_api(self, entry_id, expect):
        resp = self.cal_client.get_calibration_constant_by_id_api(entry_id)
        receive = self.get_and_validate_entry_by_id(resp)
        self.fields_validation(receive, expect)

    def __get_entry_by_uk_api(self, entry, expect):
        resp = self.cal_client.get_calibration_constant_by_uk_api(
            entry['calibration_constant']['calibration_id'],
            entry['calibration_constant']['detector_type_id'],
            entry['calibration_constant']['condition_id'])

        receive = self.load_response_content(resp)

        self.fields_validation(receive, expect)
        assert resp.status_code == HTTPStatus.OK

    def __get_entry_by_uk_api_not_found_api(self):
        calibration_id = -99
        dev_type_id = -99
        cond_id = -99

        resp = self.cal_client.get_calibration_constant_by_uk_api(
            calibration_id,
            dev_type_id,
            cond_id
        )

        self.get_and_validate_resource_not_found(resp)

    def __get_all_calibration_constants_by_conditions_api(self,
                                                          calibration_id,
                                                          detector_type_id,
                                                          condition_ids,
                                                          expect):
        resp = self.cal_client.get_all_calibration_constants_by_conds_api(
            calibration_id, detector_type_id, condition_ids)

        receive = self.load_response_content(resp)

        assert [c['name'] for c in receive] == [
            self.db_cal_const2['calibration_constant']['name'],
            self.db_cal_const1['calibration_constant']['name'],
        ]

        assert resp.status_code == HTTPStatus.OK


if __name__ == '__main__':
    unittest.main()



"""ReportTest class"""

import unittest

import pytest

from .module_base import ModuleBase
from ..common.config_test import REPORT, RESOURCE_NOT_FOUND
from ...modules.report import Report

MODULE_NAME = REPORT


@pytest.mark.usefixtures('client_cls')
class ReportTest(ModuleBase, unittest.TestCase):
    def setUp(self):
        __unique_name1 = 'Report01-Dynamic-TO_DELETED'
        self.report_01 = {
            'name': __unique_name1,
            'file_path': __unique_name1,
            'flg_available': True,
            'description': 'desc 01'
        }

        __unique_name_upd = 'Report01-Dynamic_UPD-TO_DELETED'
        self.report_01_upd = {
            'name': __unique_name_upd,
            'file_path': __unique_name_upd,
            'flg_available': False,
            'description': 'desc 01 Updated!'
        }

    def test_create_report(self):
        report_01 = Report(
            calibration_client=self.cal_client,
            name=self.report_01['name'],
            file_path=self.report_01['file_path'],
            flg_available=self.report_01['flg_available'],
            description=self.report_01['description'])

        #
        # Create new entry (should succeed)
        #
        result1 = report_01.create()

        try:
            self.assert_create_success(MODULE_NAME, result1, self.report_01)

            report_id = result1['data']['id']
            report_name = result1['data']['name']
            report_file_path = result1['data']['file_path']

            # Create duplicated entry (should throw an error)
            report_01_dup = report_01
            result2 = report_01_dup.create()
            expect_app_info = {'name': ['has already been taken'],
                               'file_path': ['has already been taken']}
            self.assert_create_error(MODULE_NAME, result2, expect_app_info)

            # Set duplicated entry
            # (should NOT throw an error and return the report from the DB)
            result3 = report_01_dup.set()
            self.assert_set_success(MODULE_NAME, result3, self.report_01)

            # Get entry by name & file_path
            result3 = Report.get_by_name_and_file_path(self.cal_client,
                                                       report_name,
                                                       report_file_path)
            self.assert_find_success(MODULE_NAME, result3, self.report_01)

            # Get entry by ID
            result4 = Report.get_by_id(self.cal_client, report_id)
            self.assert_find_success(MODULE_NAME, result4, self.report_01)

            # Get entry with non-existent ID (should throw an error)
            result5 = Report.get_by_id(self.cal_client, -666)
            self.assert_find_error(MODULE_NAME, result5, RESOURCE_NOT_FOUND)

            # Put entry information (update some fields should succeed)
            report_01.name = self.report_01_upd['name']
            report_01.file_path = self.report_01_upd['file_path']
            report_01.flg_available = self.report_01_upd['flg_available']
            report_01.description = self.report_01_upd['description']
            result6 = report_01.update()
            self.assert_update_success(MODULE_NAME, result6,
                                       self.report_01_upd)

            # Put entry information (update some fields should throw an error)
            report_01.name = '__THIS_NAME_IS_1_CHARACTERS_LONGER_THAN_THE_ALLOWED_MAX_NUM_1234__'  # noqa
            report_01.flg_available = self.report_01_upd['flg_available']
            report_01.description = self.report_01_upd['description']
            result7 = report_01.update()
            expect_app_info = {
                'name': ['is too long (maximum is 64 characters)']}
            self.assert_update_error(MODULE_NAME, result7, expect_app_info)

        finally:
            # Delete entry (should succeed)
            # (test purposes only to keep the DB clean)
            result8 = report_01.delete()

        # Check deletion worked if there isn't already an error to report
        self.assert_delete_success(MODULE_NAME, result8)

        # Delete entry (should throw an error)
        # (test purposes only to keep the DB clean)
        result9 = report_01.delete()
        self.assert_delete_error(MODULE_NAME, result9, RESOURCE_NOT_FOUND)

    def fields_validation(self, receive, expect):
        assert receive['name'] == expect['name']
        assert receive['file_path'] == expect['file_path']
        assert receive['flg_available'] == expect['flg_available']
        assert receive['description'] == expect['description']


if __name__ == '__main__':
    unittest.main()



"""BaseTest Class with helper methods common to all modules tests"""

import unittest


class ModuleBase(unittest.TestCase):
    def assert_create_success(self, module_name, receive, exp_data):
        msg = '{0} created successfully'.format(module_name)
        exp = {'success': True, 'info': msg, 'app_info': {}, 'data': exp_data}
        self.__assert_success(receive, exp)

    def assert_find_success(self, module_name, receive, exp_data):
        msg = 'Got {0} successfully'.format(module_name)
        exp = {'success': True, 'info': msg, 'app_info': {}, 'data': exp_data}
        self.__assert_success(receive, exp)

    def assert_update_success(self, module_name, receive, exp_data):
        msg = '{0} updated successfully'.format(module_name)
        exp = {'success': True, 'info': msg, 'app_info': {}, 'data': exp_data}
        self.__assert_success(receive, exp)

    def assert_delete_success(self, module_name, receive):
        msg = '{0} deleted successfully'.format(module_name)
        exp = {'success': True, 'info': msg, 'app_info': {}, 'data': {}}
        self.__assert_success(receive, exp)

    def assert_set_success(self, module_name, receive, exp_data):
        msg = '{0} set successfully'.format(module_name)
        exp = {'success': True, 'info': msg, 'app_info': {}, 'data': exp_data}
        self.__assert_success(receive, exp)

    def assert_create_error(self, module_name, receive, exp_info):
        msg = 'Error creating {0}'.format(module_name)
        exp = {'success': False, 'info': msg, 'app_info': exp_info, 'data': {}}
        self.__assert_error(receive, exp)

    def assert_find_error(self, module_name, receive, exp_info):
        msg = '{0} not found!'.format(module_name)
        exp = {'success': False, 'info': msg, 'app_info': exp_info, 'data': {}}
        self.__assert_error(receive, exp)

    def assert_update_error(self, module_name, receive, exp_info):
        msg = 'Error updating {0}'.format(module_name)
        exp = {'success': False, 'info': msg, 'app_info': exp_info, 'data': {}}
        self.__assert_error(receive, exp)

    def assert_delete_error(self, module_name, receive, exp_info):
        msg = 'Error deleting {0}'.format(module_name)
        exp = {'success': False, 'info': msg, 'app_info': exp_info, 'data': {}}
        self.__assert_error(receive, exp)

    def assert_set_error(self, module_name, receive, exp_info):
        msg = 'Error setting {0}'.format(module_name)
        exp = {'success': False, 'info': msg, 'app_info': exp_info, 'data': {}}
        self.__assert_error(receive, exp)

    def __assert_success(self, receive, expect):
        assert receive['app_info'] == {}

        # Validate all 'data' fields
        expect_data = expect['data']
        if type(expect_data) is list and len(expect_data) > 0:
            for i in range(0, len(expect_data)):
                self.fields_validation(receive['data'][i], expect['data'][i])
        elif type(expect_data) is dict and len(expect_data) > 0:
            self.fields_validation(receive['data'], expect['data'])

        assert receive['info'] == expect['info']
        assert receive['success'] == expect['success']

    def __assert_error(self, receive, expect):
        field = 'app_info'
        expect_app_info = expect[field]

        if type(expect_app_info) is dict and len(expect_app_info) > 0:
            for key, val in expect_app_info.items():
                assert receive[field][key] == expect_app_info[key]
        else:
            assert receive[field] == expect[field]

        assert receive['data'] == expect['data']
        assert receive['info'] == expect['info']
        assert receive['success'] == expect['success']



__author__ = 'Luís Maia <luis.maia@xfel.eu>'

from .calibration_api import CalibrationApi
from .calibration_constant_api import CalibrationConstantApi
from .calibration_constant_version_api import CalibrationConstantVersionApi
from .condition_api import ConditionApi
from .detector_api import DetectorApi
from .detector_type_api import DetectorTypeApi
from .parameter_api import ParameterApi
from .physical_detector_unit_api import PhysicalDetectorUnitApi
from .report_api import ReportApi
from .user_api import UserApi



"""CalibrationConstantTest class"""

import unittest

import pytest

from .module_base import ModuleBase
from ..common.config_test import CALIBRATION_CONSTANT, RESOURCE_NOT_FOUND
from ..common.generators import generate_unique_name
from ...modules.calibration_constant import CalibrationConstant

MODULE_NAME = CALIBRATION_CONSTANT


@pytest.mark.usefixtures('client_cls')
class CalibrationConstantTest(ModuleBase, unittest.TestCase):
    def setUp(self):
        __unique_name1 = generate_unique_name('CalConst01')
        self.cc_01 = {
            'name': __unique_name1,
            'calibration_id': -2,
            'detector_type_id': -1,
            'condition_id': -1,
            'flg_auto_approve': True,
            'flg_available': True,
            'description': 'desc 01'
        }

        __unique_name_upd = generate_unique_name('CalConstUpd01')
        self.cc_01_upd = {
            'name': __unique_name_upd,
            'calibration_id': -2,
            'detector_type_id': -1,
            'condition_id': -1,
            'flg_auto_approve': False,
            'flg_available': False,
            'description': 'desc 01 updated!'
        }

    def test_create_calibration_constant(self):
        cc_01 = CalibrationConstant(
            calibration_client=self.cal_client,
            name=self.cc_01['name'],
            calibration_id=self.cc_01['calibration_id'],
            detector_type_id=self.cc_01['detector_type_id'],
            condition_id=self.cc_01['condition_id'],
            flg_auto_approve=self.cc_01['flg_auto_approve'],
            flg_available=self.cc_01['flg_available'],
            description=self.cc_01['description']
        )

        # Create new entry (should succeed)
        result1 = cc_01.create()
        self.assert_create_success(MODULE_NAME, result1, self.cc_01)

        calibration_constant = result1['data']
        cc_id = calibration_constant['id']
        cc_name = calibration_constant['name']
        cc_calibration_id = calibration_constant['calibration_id']
        cc_detector_type_id = calibration_constant['detector_type_id']
        cc_condition_id = calibration_constant['condition_id']

        # Create duplicated entry (should throw an error)
        try:
            result2 = cc_01.create()
            expect_app_info = {'name': ['has already been taken'],
                               'condition': ['has already been taken'],
                               'detector_type': ['has already been taken'],
                               'calibration': ['has already been taken']}
            self.assert_create_error(MODULE_NAME, result2, expect_app_info)

            # Get entry by name
            result3 = CalibrationConstant.get_by_name(self.cal_client, cc_name)
            self.assert_find_success(MODULE_NAME, result3, self.cc_01)

            # Get entry by ID
            result4 = CalibrationConstant.get_by_id(self.cal_client, cc_id)
            self.assert_find_success(MODULE_NAME, result4, self.cc_01)

            # Get entry with non-existent ID (should throw an error)
            cc_id = -666
            result5 = CalibrationConstant.get_by_id(self.cal_client, cc_id)
            self.assert_find_error(MODULE_NAME, result5, RESOURCE_NOT_FOUND)

            # Get entry by UK (should succeed)
            # (calibration_id, detector_type_id, condition_id)
            result_uk = CalibrationConstant.get_by_uk(self.cal_client,
                                                      cc_calibration_id,
                                                      cc_detector_type_id,
                                                      cc_condition_id)
            self.assert_find_success(MODULE_NAME, result_uk, self.cc_01)

            # Get entry by UK (should throw an error)
            # (calibration_id, detector_type_id, condition_id)
            calibration_id = -666
            detector_type_id = -666
            condition_id = -666
            res_uk_error = CalibrationConstant.get_by_uk(
                self.cal_client, calibration_id, detector_type_id, condition_id
            )
            self.assert_find_error(MODULE_NAME, res_uk_error,
                                   RESOURCE_NOT_FOUND)

            # Put entry information (update some fields should succeed)
            cc_01.name = self.cc_01_upd['name']
            cc_01.flg_auto_approve = self.cc_01_upd['flg_auto_approve']
            cc_01.flg_available = self.cc_01_upd['flg_available']
            cc_01.description = self.cc_01_upd['description']
            result6 = cc_01.update()
            self.assert_update_success(MODULE_NAME, result6, self.cc_01_upd)

            # Put entry information (update some fields should throw an error)
            wrong_name = "__THIS_NAME_IS_1_CHARACTERS_LONGER_THAN"  # 39 char
            wrong_name += "_THE_ALLOWED_MAX_NUM__"  # 22 char
            wrong_name += "_(NUM_CHARACTERS_IS_256)_"  # 25 char
            # 256 - (39 + 22 + 25) =
            wrong_name += "-->_{0}".format('Z' * 170)

            cc_01.name = wrong_name
            cc_01.flg_available = self.cc_01_upd['flg_available']
            cc_01.description = self.cc_01_upd['description']
            result7 = cc_01.update()

            expect_app_info = {
                'name': ['is too long (maximum is 255 characters)']}
            self.assert_update_error(MODULE_NAME, result7, expect_app_info)

        finally:
            # Delete entry (should succeed)
            # (test purposes only to keep the DB clean)
            result8 = cc_01.delete()

        # Check deletion worked if there isn't already an error to report
        self.assert_delete_success(MODULE_NAME, result8)

        # Delete entry (should throw an error)
        # (test purposes only to keep the DB clean)
        result9 = cc_01.delete()
        self.assert_delete_error(MODULE_NAME, result9, RESOURCE_NOT_FOUND)

    def test_create_calibration_constant_from_dict(self):
        # Create new entry (should succeed)
        result1 = CalibrationConstant.set_from_dict(self.cal_client,
                                                    self.cc_01)
        self.assert_create_success(MODULE_NAME, result1, self.cc_01)

        calibration_constant = result1['data']
        cc_id = calibration_constant['id']

        try:
            # Create duplicated entry (should throw an error)
            result2 = CalibrationConstant.create_from_dict(self.cal_client,
                                                           self.cc_01)
            expect_app_info = {'name': ['has already been taken'],
                               'condition': ['has already been taken'],
                               'detector_type': ['has already been taken'],
                               'calibration': ['has already been taken']}
            self.assert_create_error(MODULE_NAME, result2, expect_app_info)

            # Set existing entry (should find item successfully)
            result3 = CalibrationConstant.set_from_dict(self.cal_client,
                                                        self.cc_01)
            self.assert_find_success(MODULE_NAME, result3, self.cc_01)

            # Get entry by UK (should succeed)
            # (calibration_id, detector_type_id, condition_id)
            result_uk = CalibrationConstant.set_from_dict(self.cal_client,
                                                          self.cc_01)
            self.assert_find_success(MODULE_NAME, result_uk, self.cc_01)

        finally:
            # Delete entry (should succeed)
            # (test purposes only to keep the DB clean)
            result8 = CalibrationConstant.delete_by_id(self.cal_client, cc_id)

        # Check deletion worked if there isn't already an error to report
        self.assert_delete_success(MODULE_NAME, result8)

        # Delete entry (should throw an error)
        # (test purposes only to keep the DB clean)
        result9 = CalibrationConstant.delete_by_id(self.cal_client, cc_id)
        self.assert_delete_error(MODULE_NAME, result9, RESOURCE_NOT_FOUND)

    def fields_validation(self, receive, expect):
        assert receive['name'] == expect['name']
        assert receive['detector_type_id'] == expect['detector_type_id']
        assert receive['calibration_id'] == expect['calibration_id']
        assert receive['condition_id'] == expect['condition_id']
        assert receive['flg_auto_approve'] == expect['flg_auto_approve']
        assert receive['flg_available'] == expect['flg_available']
        assert receive['description'] == expect['description']


if __name__ == '__main__':
    unittest.main()



"""Test utility functions"""


def debug_response(response):
    print('*' * 100)
    print('response.status_code =>', str(response.status_code))
    print('response.content =>', str(response.content))
    print('*' * 100)

    # Raise Error to show response content
    raise NameError('See response message content')



"""Client library for encapsulating and managing the interaction with the
Calibration Constants Catalogue Web Application"""

__author__ = 'Luís Maia <luis.maia@xfel.eu>'
__version__ = '11.0.0'

from .calibration_client import CalibrationClient



"""CalibrationClient class"""

import logging
from time import gmtime
from time import strftime

from .apis import (
    CalibrationApi, CalibrationConstantApi, CalibrationConstantVersionApi,
    ConditionApi, ParameterApi, DetectorApi, DetectorTypeApi,
    PhysicalDetectorUnitApi, ReportApi,
    UserApi,
)
from .common.util import Util
from .modules import (
    Calibration, CalibrationConstant, CalibrationConstantVersion,
    Condition, Parameter, Detector, DetectorType, PhysicalDetectorUnit,
    Report,
)


class CalibrationClient(CalibrationApi,
                        CalibrationConstantApi,
                        CalibrationConstantVersionApi,
                        ConditionApi, ParameterApi,
                        DetectorApi, DetectorTypeApi, PhysicalDetectorUnitApi,
                        ReportApi, UserApi):

    # The parent classes define methods to make requests to different parts
    # of the API, e.g. .get_calibration_constant_version_by_id_api()
    # They all inherit from .common.base.Base which handles the oauth session.

    # Wrapper methods which may make several API calls: -------------------

    def set_calibration_constant(self, condition, cc_h):
        logging.debug('condition: {0}'.format(condition))
        logging.debug('calibration_constant: {0}'.format(cc_h))

        #
        calibration_name = cc_h['calibration_name']
        resp = Calibration.get_by_name(self, calibration_name)
        calibration_id = resp['data']['id']

        detector_type_name = cc_h['detector_type_name']
        resp = DetectorType.get_by_name(self, detector_type_name)
        detector_type_id = resp['data']['id']

        condition_name = condition['name']
        condition_id = condition['id']

        # Generate unique name if it doesn't exist
        cc_name_01 = '{0}_{1}'.format(detector_type_name, calibration_name)
        cc_name_uk = '{0}_{1}'.format(cc_name_01[:40], condition_name)
        cc_name = Util.get_opt_hash_val(cc_h, 'name', cc_name_uk)

        #
        cc_desc = Util.get_opt_hash_val(cc_h, 'description')
        cc_flg_auto_approve = Util.val_to_api_bool(cc_h['flg_auto_approve'])
        cc_flg_avail = Util.get_opt_hash_val(cc_h, 'flg_available', 'true')

        cal_cc = {
            'name': cc_name,
            'calibration_id': str(calibration_id),
            'detector_type_id': str(detector_type_id),
            'condition_id': str(condition_id),
            'flg_auto_approve': cc_flg_auto_approve,
            'flg_available': cc_flg_avail,
            'description': cc_desc
        }
        logging.debug('Built Calibration CC: {0}'.format(cal_cc))

        #
        # SET cc from dictionary
        #
        return CalibrationConstant.set_from_dict(self, cal_cc)

    def get_physical_detector_unit(self, cal_dict):
        #
        # Expected keys:
        #
        # MODE: by_physical_detector_unit_name
        # 1) cal_dict['pdu_physical_name']
        #
        #   -- OR --
        #
        # MODE: by_detector_id_and_karabo_da
        # 2) cal_dict['detector_identifier']
        # 2) cal_dict['pdu_karabo_da']
        #
        if 'pdu_physical_name' in cal_dict.keys():
            if cal_dict['pdu_physical_name'] == '' or \
                    cal_dict['pdu_physical_name'] is None:
                mode = 'by_detector_id_and_karabo_da'
            else:
                mode = 'by_physical_detector_unit_name'
        else:
            mode = 'by_detector_id_and_karabo_da'

        # Taking action accordingly to the mode
        if mode == 'by_physical_detector_unit_name':
            pdu_physical_name = cal_dict['pdu_physical_name']
            logging.debug(
                'phy_det_unit.physical_name: {0}'.format(pdu_physical_name))
            logging.debug('detector_identifier: NOT_EVALUATED')
            logging.debug('phy_det_unit.karabo_da: NOT_EVALUATED')

            resp = PhysicalDetectorUnit.get_by_name(self, pdu_physical_name)

            if resp['success']:
                physical_detector_unit = resp['data']

                success_msg = 'phy_det_unit.id: {0}'.format(
                    physical_detector_unit['id'])
                logging.debug(success_msg)

                success_msg = 'phy_det_unit.detector_type_id: {0}'.format(
                    physical_detector_unit['detector_type_id'])
                logging.debug(success_msg)

                success_msg = 'phy_det_unit.karabo_da: {0}'.format(
                    physical_detector_unit['karabo_da'])
                logging.debug(success_msg)
            else:
                error_msg = '{0} >> {1}'.format(resp['info'], resp['app_info'])
                logging.error(error_msg)
                return resp

        else:
            #
            # Get detector_id from detector_identifier
            #
            det_identifier = cal_dict['detector_identifier']
            phy_det_unit_karabo_da = cal_dict['pdu_karabo_da']

            logging.debug('phy_det_unit.physical_name: None')
            logging.debug('detector_identifier: {0}'.format(det_identifier))
            logging.debug(
                'phy_det_unit.karabo_da: {0}'.format(phy_det_unit_karabo_da))

            resp = Detector.get_by_identifier(self, det_identifier)
            if resp['success']:
                detector_id = resp['data']['id']

                success_msg = 'detector_id: {0}'.format(detector_id)
                logging.debug(success_msg)
            else:
                error_msg = '{0} >> {1}'.format(resp['info'], resp['app_info'])
                logging.error(error_msg)
                return resp

            #
            # Get physical_detector_unit_id
            #   from detector_id AND physical_detector_unit.karabo_da
            #
            resp = PhysicalDetectorUnit.get_all_by_detector_and_karabo_da(
                self, detector_id, phy_det_unit_karabo_da)

            if resp['success']:
                physical_detector_unit = resp['data'][0]

                success_msg = 'phy_det_unit.id: {0}'.format(
                    physical_detector_unit['id'])
                logging.debug(success_msg)

                success_msg = 'phy_det_unit.detector_type_id: {0}'.format(
                    physical_detector_unit['detector_type_id'])
                logging.debug(success_msg)

                success_msg = 'phy_det_unit.karabo_da: {0}'.format(
                    physical_detector_unit['karabo_da'])
                logging.debug(success_msg)
            else:
                error_msg = '{0} >> {1}'.format(resp['info'], resp['app_info'])
                logging.error(error_msg)
                return resp

        return physical_detector_unit

    def set_calibration_constant_version(self, calibration_constant_id,
                                         ccv_h, report_id):
        logging.debug('calib_const_id: {0}'.format(calibration_constant_id))
        logging.debug('Calibration Constant Version Dict: {0}'.format(ccv_h))
        logging.debug('report_id: {0}'.format(report_id))

        # Generate unique name if it doesn't exist
        datetime_str = strftime('%Y%m%d_%H%M%S', gmtime())
        cc_name_uk = '{0}_sIdx={1}'.format(datetime_str, ccv_h['start_idx'])
        cc_name = Util.get_opt_hash_val(ccv_h, 'name', cc_name_uk)

        #
        ccv_desc = Util.get_opt_hash_val(ccv_h, 'description')
        ccv_flg_good_quality = Util.val_to_api_bool(ccv_h['flg_good_quality'])
        ccv_flg_deployed = Util.get_opt_hash_val(ccv_h, 'flg_deployed',
                                                 def_val='true')

        physical_detector_unit = self.get_physical_detector_unit(ccv_h)

        cal_ccv = {
            'name': cc_name,
            'file_name': ccv_h['file_name'],
            'path_to_file': ccv_h['path_to_file'],
            'data_set_name': ccv_h['data_set_name'],
            'calibration_constant_id': str(calibration_constant_id),
            'physical_detector_unit_id': str(physical_detector_unit['id']),
            'flg_deployed': ccv_flg_deployed,
            'flg_good_quality': ccv_flg_good_quality,
            'begin_validity_at': ccv_h['begin_validity_at'],
            'end_validity_at': ccv_h['end_validity_at'],
            'begin_at': ccv_h['begin_at'],
            'start_idx': ccv_h['start_idx'],
            'end_idx': ccv_h['end_idx'],
            'raw_data_location': ccv_h['raw_data_location'],
            'report_id': str(report_id),
            'description': ccv_desc
        }
        logging.debug('Built Calibration CCV: {0}'.format(cal_ccv))

        #
        # Create new ccv from dictionary
        #
        return CalibrationConstantVersion.create_from_dict(self, cal_ccv)

    def set_condition_from_dict(self, detector_condition):
        cal_cond = self.get_condition_from_dict(detector_condition)

        return Condition.set_condition_from_dict(self, cal_cond)

    def search_condition_from_dict(self, event_at, detector_condition):
        cal_cond = self.get_condition_from_dict(detector_condition)

        cond = Condition(self,
                         cal_cond['name'],
                         cal_cond['flg_available'],
                         event_at,
                         cal_cond['parameters_conditions_attributes'],
                         cal_cond['description'])

        resp = cond.get_expected()
        if resp['success']:
            return resp
        else:
            resp = cond.get_possible()

            if resp['success']:
                # This method returns several conditions over time
                # ordered by the closest condition creation datetime,
                # compared with the desired event datetime!
                # The closest condition is received in the first position!
                resp['data'] = resp['data'][0]

            return resp

    def search_possible_conditions_from_dict(self,
                                             event_at,
                                             detector_condition):
        cal_cond = self.get_condition_from_dict(detector_condition)

        cond = Condition(self,
                         cal_cond['name'],
                         cal_cond['flg_available'],
                         event_at,
                         cal_cond['parameters_conditions_attributes'],
                         cal_cond['description'])

        resp = cond.get_possible()
        # This method returns several conditions over time
        # ordered by the closest condition creation datetime,
        # compared with the desired event datetime!

        return resp

    def get_condition_from_dict(self, detector_condition):
        parameters_conditions = []

        for param in detector_condition['parameters']:
            param_name = param['parameter_name']
            param_desc = Util.get_opt_hash_val(param, 'description')

            resp = Parameter.get_by_name(self, param_name)
            param_id = resp['data']['id']

            parameter_condition = {
                'parameter_id': str(param_id),
                'value': str(param['value']),
                'lower_deviation_value': str(param['lower_deviation_value']),
                'upper_deviation_value': str(param['upper_deviation_value']),
                'flg_available': Util.val_to_api_bool(param['flg_available']),
                'description': str(param_desc)
            }

            logging.debug('Build parameter_condition hash successfully')
            parameters_conditions.append(parameter_condition)

        def_cond_name = strftime('%Y-%m-%d %H:%M:%S', gmtime())
        condition_name = Util.get_opt_hash_val(detector_condition, 'name',
                                               def_val=def_cond_name)

        condition_flg_avail = Util.get_opt_hash_val(detector_condition,
                                                    'flg_available',
                                                    def_val='true')
        condition_desc = Util.get_opt_hash_val(detector_condition,
                                               'description')

        #
        cal_cond = {
            'name': condition_name,
            'flg_available': condition_flg_avail,
            'description': condition_desc,
            'parameters_conditions_attributes': parameters_conditions
        }
        logging.debug('Build condition successfully: {0}'.format(cal_cond))

        return cal_cond

    def set_report(self, report_h):
        logging.debug('* report_h == {0}'.format(report_h))

        rep_name = report_h['name']
        rep_path = report_h['file_path']
        rep_flg_available = True
        rep_description = Util.get_opt_hash_val(report_h, 'description')

        resp = Report.get_by_name_and_file_path(self, rep_name, rep_path)

        if resp['success']:
            report = resp['data']

            if report != []:
                report_id = report['id']

                success_msg = 'FOUND report_id: {0}'.format(report_id)
                logging.debug(success_msg)

                return resp

        # In case no data found
        report = Report(self, rep_name, rep_path,
                        rep_flg_available, rep_description)

        resp = report.set()

        if resp['success']:
            report = resp['data']
            report_id = report['id']

            success_msg = 'CREATED report_id: {0}'.format(report_id)
            logging.debug(success_msg)

            return resp

        else:
            error_msg = '{0} >> {1}'.format(resp['info'],
                                            resp['app_info'])
            logging.error(error_msg)
            return resp

    def get_all_phy_det_units_from_detector(self, det_retrieve_h):
        logging.debug('* det_retrieve_h == {0}'.format(det_retrieve_h))

        det_identifier = det_retrieve_h['detector_identifier']
        snapshot_at = Util.get_opt_hash_val(det_retrieve_h, 'snapshot_at')

        resp = self.get_all_phy_det_units_from_detector_int(det_identifier,
                                                            snapshot_at)

        if not resp['success']:
            error_msg = '{0} >> {1}'.format(resp['info'], resp['app_info'])
            logging.error(error_msg)
            return resp

        return resp

    def get_all_phy_det_units_from_detector_int(self, detector_identifier,
                                                snapshot_at):
        #
        # Get detector_id from detector_identifier
        #
        logging.debug('* det_identifier == {0}'.format(detector_identifier))
        logging.debug('* snapshot_at == {0}'.format(snapshot_at))

        resp = Detector.get_by_identifier(self, detector_identifier)
        if resp['success']:
            detector_id = resp['data']['id']

            success_msg = 'detector_id == {0}'.format(detector_id)
            logging.debug(success_msg)
        else:
            error_msg = '{0} >> {1}'.format(resp['info'], resp['app_info'])
            logging.error(error_msg)
            return resp

        # get_all_by_detector
        resp = PhysicalDetectorUnit.get_all_by_detector(self, detector_id,
                                                        snapshot_at)
        if resp['success']:
            all_phy_det_units = resp['data']

            phy_det_unit_id_0 = all_phy_det_units[0]['id']
            detector_type_id_0 = all_phy_det_units[0]['detector_type_id']
            detector_id_0 = all_phy_det_units[0]['detector_id']

            success_msg = 'phy_det_unit_id_0 == {0}'.format(phy_det_unit_id_0)
            logging.debug(success_msg)
            success_msg = 'det_type_id_0 == {0}'.format(detector_type_id_0)
            logging.debug(success_msg)
            success_msg = 'detector_id_0 == {0}'.format(detector_id_0)
            logging.debug(success_msg)
        else:
            error_msg = '{0} >> {1}'.format(resp['info'], resp['app_info'])
            logging.error(error_msg)
            return resp

        return resp

    def inject_new_calibration_constant_version(self, inject_h):
        logging.debug('Calibration injection hash is: {0}'.format(inject_h))

        # Separate the inject_h into its main elements
        if 'report' in inject_h:
            inj_report_info = inject_h['report']
        else:
            inj_report_info = {}

        inj_detector_condition = inject_h['detector_condition']
        inj_cal_const = inject_h['calibration_constant']
        inj_cal_const_version = inject_h['calibration_constant_version']

        # set_condition
        resp = self.set_condition_from_dict(inj_detector_condition)

        if resp['success']:
            condition = resp['data']
            condition_id = condition['id']

            success_msg = 'condition_id: {0}'.format(condition_id)
            logging.debug(success_msg)
        else:
            error_msg = '{0} >> {1}'.format(resp['info'], resp['app_info'])
            logging.error(error_msg)
            return resp

        # set_calibration_constant
        resp = self.set_calibration_constant(condition, inj_cal_const)
        if resp['success']:
            report = resp['data']
            cc_id = report['id']

            success_msg = 'condition_id: {0}'.format(cc_id)
            logging.debug(success_msg)
        else:
            error_msg = '{0} >> {1}'.format(resp['info'], resp['app_info'])
            logging.error(error_msg)
            return resp

        # Create Report entity only if received
        report_id = ''

        if inj_report_info != {}:
            resp = self.set_report(inj_report_info)
            if resp['success']:
                report = resp['data']
                report_id = report['id']

                success_msg = 'report_id: {0}'.format(report_id)
                logging.debug(success_msg)
            else:
                error_msg = '{0} >> {1}'.format(resp['info'], resp['app_info'])
                logging.error(error_msg)
                return resp

        # create_calibration_constant_version
        resp = self.set_calibration_constant_version(
            cc_id, inj_cal_const_version, report_id
        )

        if resp['success']:
            cal_calibration_constant_version = resp['data']
            ccv_id = cal_calibration_constant_version['id']

            success_msg = 'condition_id: {0}'.format(ccv_id)
            logging.debug(success_msg)
        else:
            error_msg = '{0} >> {1}'.format(resp['info'], resp['app_info'])
            logging.error(error_msg)
            return resp

        # In case of success
        return resp

    def retrieve_calibration_constant_version(self, retrieve_h):
        logging.debug('Calibration retrieve hash is: {0}'.format(retrieve_h))

        # Retrieve the desired retrieve Strategy
        # OPTIONS:
        #   - pdu_closest_by_time (default)
        #   - pdu_prior_in_time
        #   - detector_closest_by_time
        if 'strategy' in retrieve_h:
            ret_strategy = retrieve_h['strategy']
        else:
            ret_strategy = 'pdu_closest_by_time'

        # Call proper method depending on the desired 'Strategy'
        if ret_strategy == 'pdu_prior_in_time':
            resp = self.retrieve_calibration_constant_version_by_pdu(
                ret_strategy, retrieve_h)

        elif ret_strategy == 'detector_closest_by_time':
            resp = self.retrieve_calibration_constant_version_by_detector(
                ret_strategy, retrieve_h)

        # if ret_strategy == '' or
        #    ret_strategy is None or
        #    ret_strategy == 'pdu_closest_by_time':
        else:
            resp = self.retrieve_calibration_constant_version_by_pdu(
                ret_strategy, retrieve_h)

        return resp

    def retrieve_calibration_constant_version_by_pdu(self,
                                                     ret_strategy,
                                                     retrieve_h):
        logging.debug('CCV retrieve strategy: {0}'.format(ret_strategy))
        logging.debug('Calibration retrieve hash is: {0}'.format(retrieve_h))

        # Retrieve the desired retrieve Strategy
        # OPTIONS:
        #  - pdu_closest_by_time (default)
        #  - pdu_prior_in_time
        #  - detector_closest_by_time [ERROR! <= Not available in this method!]

        # Separate the inject_h into its main elements
        ret_detector_condition = retrieve_h['detector_condition']

        # Calculate necessary values
        calibration_name = retrieve_h['calibration_name']
        event_at = Util.get_opt_hash_val(retrieve_h, 'measured_at')
        snapshot_at = Util.get_opt_hash_val(retrieve_h, 'snapshot_at')

        # get_condition a valid condition
        resp = self.search_possible_conditions_from_dict(
            event_at, ret_detector_condition
        )

        if resp['success']:
            conditions = resp['data']

            condition_ids = []
            for condition in conditions:
                condition_ids.append(condition['id'])

            success_msg = 'condition_ids: {0}'.format(condition_ids)
            logging.debug(success_msg)
        else:
            error_msg = '{0} >> {1}'.format(resp['info'], resp['app_info'])
            logging.error(error_msg)
            return resp

        # Get Calibration ID
        resp = Calibration.get_by_name(self, calibration_name)
        if resp['success']:
            calibration = resp['data']
            calibration_id = calibration['id']

            success_msg = 'condition_id: {0}'.format(calibration_id)
            logging.debug(success_msg)
        else:
            error_msg = '{0} >> {1}'.format(resp['info'], resp['app_info'])
            logging.error(error_msg)
            return resp

        # Get PhysicalDetectorUnit ID and DetectorType ID
        physical_detector_unit = self.get_physical_detector_unit(retrieve_h)

        phy_det_unit_id = physical_detector_unit['id']
        detector_type_id = physical_detector_unit['detector_type_id']

        # Get CalibrationConstant ID
        resp = CalibrationConstant.get_all_by_conditions(self,
                                                         calibration_id,
                                                         detector_type_id,
                                                         condition_ids)
        if resp['success']:
            calibration_constants = resp['data']

            cc_ids = []
            for cc in calibration_constants:
                cc_ids.append(cc['id'])

            success_msg = 'calibration_constant_ids: {0}'.format(cc_ids)
            logging.debug(success_msg)
        else:
            error_msg = '{0} >> {1}'.format(resp['info'], resp['app_info'])
            logging.error(error_msg)
            return resp

        # Debug IDs
        logging.debug('* ret_strategy == {0}'.format(ret_strategy))
        logging.debug('* condition_ids == {0}'.format(condition_ids))
        logging.debug('* calibration_id == {0}'.format(calibration_id))
        logging.debug('* phy_det_unit_id == {0}'.format(phy_det_unit_id))
        logging.debug('* detector_type_id == {0}'.format(detector_type_id))
        logging.debug('* calibration_constant_ids == {0}'.format(cc_ids))

        # Get calibration_constant_version
        if ret_strategy == 'pdu_prior_in_time':
            # resp = CalibrationConstantVersion.get_by_uk(self,
            resp = CalibrationConstantVersion.get_prior_in_time(self,
                                                                cc_ids,
                                                                phy_det_unit_id,
                                                                event_at,
                                                                snapshot_at)

        elif ret_strategy == 'detector_closest_by_time':
            error_msg = 'Strategy "detector_closest_by_time" not available ' \
                        'in retrieve_calibration_constant_version'
            logging.error(error_msg)
            return resp

        # if ret_strategy == '' or
        #    ret_strategy is None or
        #    ret_strategy == 'pdu_closest_by_time':
        else:
            resp = CalibrationConstantVersion.get_closest_by_time(
                self,
                cc_ids, phy_det_unit_id,
                event_at, snapshot_at)

        if resp['success']:
            calibration_constant_version = resp['data']
            ccv_id = calibration_constant_version['id']

            success_msg = 'calibration_constant_version_id: {0}'.format(ccv_id)
            logging.debug(success_msg)
        else:
            error_msg = '{0} >> {1}'.format(resp['info'], resp['app_info'])
            logging.error(error_msg)
            return resp

        # In case of success
        return resp

    def retrieve_calibration_constant_version_by_detector(self,
                                                          ret_strategy,
                                                          retrieve_h):
        logging.debug('CCV retrieve strategy: {0}'.format(ret_strategy))
        logging.debug('Calibration retrieve hash is: {0}'.format(retrieve_h))

        # Retrieve the desired retrieve Strategy
        # OPTIONS:
        #  - pdu_closest_by_time  [ERROR! <= Not available in this method!]
        #  - pdu_prior_in_time [ERROR! <= Not available in this method!]
        #  - detector_closest_by_time (default)

        # Separate the inject_h into its main elements
        ret_detector_condition = retrieve_h['detector_condition']

        # Calculate necessary values
        calibration_name = retrieve_h['calibration_name']
        event_at = Util.get_opt_hash_val(retrieve_h, 'measured_at')
        snapshot_at = Util.get_opt_hash_val(retrieve_h, 'snapshot_at')

        # get_condition a valid condition
        resp = self.search_possible_conditions_from_dict(
            event_at, ret_detector_condition
        )

        if resp['success']:
            conditions = resp['data']

            condition_ids = []
            for condition in conditions:
                condition_ids.append(condition['id'])

            success_msg = 'condition_ids: {0}'.format(condition_ids)
            logging.debug(success_msg)
        else:
            error_msg = '{0} >> {1}'.format(resp['info'], resp['app_info'])
            logging.error(error_msg)
            return resp

        # Get Calibration ID
        resp = Calibration.get_by_name(self, calibration_name)
        if resp['success']:
            calibration = resp['data']
            calibration_id = calibration['id']

            success_msg = 'calibration_id: {0}'.format(calibration_id)
            logging.debug(success_msg)
        else:
            error_msg = '{0} >> {1}'.format(resp['info'], resp['app_info'])
            logging.error(error_msg)
            return resp

        #
        # Get detector_id from detector_identifier
        #
        det_identifier = retrieve_h['detector_identifier']
        logging.debug('* detector_identifier == {0}'.format(det_identifier))

        resp = self.get_all_phy_det_units_from_detector_int(det_identifier,
                                                            snapshot_at)

        if resp['success']:
            all_phy_det_units = resp['data']

            phy_det_unit_id_0 = all_phy_det_units[0]['id']
            detector_type_id = all_phy_det_units[0]['detector_type_id']
            detector_id = all_phy_det_units[0]['detector_id']

            success_msg = 'all_phy_det_units == {0}'.format(all_phy_det_units)
            logging.debug(success_msg)
        else:
            error_msg = '{0} >> {1}'.format(resp['info'], resp['app_info'])
            logging.error(error_msg)
            return resp

        # Get CalibrationConstant ID
        resp = CalibrationConstant.get_all_by_conditions(self,
                                                         calibration_id,
                                                         detector_type_id,
                                                         condition_ids)
        if resp['success']:
            calibration_constants = resp['data']

            cc_ids = []
            for cc in calibration_constants:
                cc_ids.append(cc['id'])

            success_msg = 'calibration_constant_ids: {0}'.format(cc_ids)
            logging.debug(success_msg)
        else:
            error_msg = '{0} >> {1}'.format(resp['info'], resp['app_info'])
            logging.error(error_msg)
            return resp

        # Debug IDs
        logging.debug('* ret_strategy == {0}'.format(ret_strategy))
        logging.debug('* condition_ids == {0}'.format(condition_ids))
        logging.debug('* calibration_id == {0}'.format(calibration_id))
        logging.debug('* detector_type_id == {0}'.format(detector_type_id))
        logging.debug('* detector_id == {0}'.format(detector_id))
        logging.debug('* calibration_constant_ids == {0}'.format(cc_ids))
        logging.debug('* phy_det_units == {0}'.format(all_phy_det_units))

        # Get calibration_constant_version
        if ret_strategy == 'pdu_prior_in_time':
            error_msg = 'Strategy "pdu_prior_in_time" not available' \
                        'in retrieve_calibration_constant_version_by_detector'
            logging.error(error_msg)
            return resp

        elif ret_strategy == 'pdu_closest_by_time':
            error_msg = 'Strategy "pdu_closest_by_time" not available in' \
                        ' retrieve_calibration_constant_version_by_detector'
            logging.error(error_msg)
            return resp

        # if ret_strategy == '' or
        #    ret_strategy is None or
        #    ret_strategy == 'detector_closest_by_time':
        else:
            karabo_da = Util.get_opt_hash_val(retrieve_h, 'pdu_karabo_da')
            logging.debug('* karabo_da == {0}'.format(karabo_da))

            resp = CalibrationConstantVersion.get_closest_by_time_by_detector(
                self,
                cc_ids, detector_id, karabo_da,
                event_at, snapshot_at)

        if resp['success']:
            # Override resp['data'] to return an object, instead of an Array
            # as returned by all the other scenarios
            resp['data'] = resp['data'][0]
            calibration_constant_version = resp['data']
            ccv_id = calibration_constant_version['id']

            success_msg = 'calibration_constant_version_id: {0}'.format(ccv_id)
            logging.debug(success_msg)
        else:
            error_msg = '{0} >> {1}'.format(resp['info'], resp['app_info'])
            logging.error(error_msg)
            return resp

        # In case of success
        return resp

    def retrieve_all_calibration_constant_versions(self, retrieve_h):
        logging.debug('Calibration retrieve hash is: {0}'.format(retrieve_h))

        # Separate the inject_h into its main elements
        ret_detector_condition = retrieve_h['detector_condition']

        # Calculate necessary values
        calibration_name = retrieve_h['calibration_name']
        event_at = Util.get_opt_hash_val(retrieve_h, 'measured_at')
        snapshot_at = Util.get_opt_hash_val(retrieve_h, 'snapshot_at')

        # get_condition a valid condition
        resp = self.search_possible_conditions_from_dict(
            event_at, ret_detector_condition,
        )

        if resp['success']:
            conditions = resp['data']

            condition_ids = []
            for condition in conditions:
                condition_ids.append(condition['id'])

            success_msg = 'condition_ids: {0}'.format(condition_ids)
            logging.debug(success_msg)
        else:
            error_msg = '{0} >> {1}'.format(resp['info'], resp['app_info'])
            logging.error(error_msg)
            return resp

        # Get Calibration ID
        resp = Calibration.get_by_name(self, calibration_name)
        if resp['success']:
            calibration = resp['data']
            calibration_id = calibration['id']

            success_msg = 'condition_id: {0}'.format(calibration_id)
            logging.debug(success_msg)
        else:
            error_msg = '{0} >> {1}'.format(resp['info'], resp['app_info'])
            logging.error(error_msg)
            return resp

        # Get PhysicalDetectorUnit ID and DetectorType ID
        physical_detector_unit = self.get_physical_detector_unit(retrieve_h)

        phy_det_unit_id = physical_detector_unit['id']
        detector_type_id = physical_detector_unit['detector_type_id']

        # Get CalibrationConstant ID
        resp = CalibrationConstant.get_all_by_conditions(self,
                                                         calibration_id,
                                                         detector_type_id,
                                                         condition_ids)
        if resp['success']:
            calibration_constants = resp['data']

            cc_ids = []
            for cc in calibration_constants:
                cc_ids.append(cc['id'])

            success_msg = 'calibration_constant_ids: {0}'.format(cc_ids)
            logging.debug(success_msg)
        else:
            error_msg = '{0} >> {1}'.format(resp['info'], resp['app_info'])
            logging.error(error_msg)
            return resp

        # Debug IDs
        logging.debug('* condition_ids == {0}'.format(condition_ids))
        logging.debug('* calibration_id == {0}'.format(calibration_id))
        logging.debug('* phy_det_unit_id == {0}'.format(phy_det_unit_id))
        logging.debug('* detector_type_id == {0}'.format(detector_type_id))
        logging.debug('* calibration_constant_ids == {0}'.format(cc_ids))

        # Get all calibration_constant_versions
        resp = CalibrationConstantVersion.get_all_versions(self,
                                                           cc_ids,
                                                           phy_det_unit_id,
                                                           event_at,
                                                           snapshot_at)
        if resp['success']:
            success_msg = 'response successful!'
            logging.debug(success_msg)
        else:
            error_msg = '{0} >> {1}'.format(resp['info'], resp['app_info'])
            logging.error(error_msg)
            return resp

        # In case of success
        return resp

    def update_calibration_constant_version(self, update_h):
        logging.debug('Calibration update hash is: {0}'.format(update_h))

        # Calculate necessary values
        ccv_id = update_h['ccv_id']

        # Get calibration_constant_version
        resp = CalibrationConstantVersion.get_by_id(self, ccv_id)
        if resp['success']:
            ccv = resp['data']
            success_msg = 'retrieve_constant_version_id: {0}'.format(ccv_id)
            logging.debug(success_msg)
        else:
            error_msg = '{0} >> {1}'.format(resp['info'], resp['app_info'])
            logging.error(error_msg)
            return resp

        # Update calibration_constant_version
        # set parameters from DB
        ccv_pars = {k: v for k, v in ccv.items() if
                    (not isinstance(v, dict) and k != 'id')}
        ccv_pars['calibration_constant_id'] = ccv['calibration_constant']['id']
        ccv_pars['physical_detector_unit_id'] = ccv['physical_detector_unit'][
            'id']

        # Update parameters
        updatable = ['description', 'end_idx', 'start_idx', 'begin_at',
                     'end_validity_at', 'begin_validity_at',
                     'flg_good_quality', 'flg_deployed']

        for key in updatable:
            if key in update_h:
                ccv_pars[key] = update_h[key]

        new_ccv = CalibrationConstantVersion(
            calibration_client=self,
            **ccv_pars)

        new_ccv.id = ccv_id
        resp = new_ccv.update()

        if resp['success']:
            success_msg = 'update_constant_version_id: {0}'.format(ccv_id)
            logging.debug(success_msg)
        else:
            error_msg = '{0} >> {1}'.format(resp['info'], resp['app_info'])
            logging.error(error_msg)
            return resp

        # In case of success
        return resp



"""CalibrationConstant module class"""

import logging
from http import HTTPStatus

from ..common.base import Base
from ..common.config import *

MODULE_NAME = CALIBRATION_CONSTANT


class CalibrationConstant:
    def __init__(self, calibration_client,
                 name, calibration_id, detector_type_id, condition_id,
                 flg_available, flg_auto_approve,
                 description=''):
        self.calibration_client = calibration_client
        self.id = None
        self.name = name
        self.calibration_id = calibration_id
        self.detector_type_id = detector_type_id
        self.condition_id = condition_id
        self.flg_available = flg_available
        self.flg_auto_approve = flg_auto_approve
        self.description = description

    def create(self):
        cal_client = self.calibration_client
        response = cal_client.create_calibration_constant_api(
            self.__get_resource())

        Base.cal_debug(MODULE_NAME, CREATE, response)
        res = Base.format_response(
            response, CREATE, HTTPStatus.CREATED, MODULE_NAME
        )

        if res['success']:
            self.id = res['data']['id']

        return res

    def delete(self):
        cal_client = self.calibration_client
        response = cal_client.delete_calibration_constant_api(self.id)
        Base.cal_debug(MODULE_NAME, DELETE, response)

        return Base.format_response(
            response, DELETE, HTTPStatus.NO_CONTENT, MODULE_NAME
        )

    def update(self):
        cal_client = self.calibration_client
        response = cal_client.update_calibration_constant_api(
            self.id,
            self.__get_resource()
        )

        Base.cal_debug(MODULE_NAME, UPDATE, response)
        return Base.format_response(
            response, UPDATE, HTTPStatus.OK, MODULE_NAME
        )

    @staticmethod
    def create_from_dict(cal_client, cc):
        new_cc = CalibrationConstant(calibration_client=cal_client,
                                     name=cc['name'],
                                     calibration_id=cc['calibration_id'],
                                     detector_type_id=cc['detector_type_id'],
                                     condition_id=cc['condition_id'],
                                     flg_auto_approve=cc['flg_auto_approve'],
                                     flg_available=cc['flg_available'],
                                     description=cc['description'])

        resp = new_cc.create()
        return resp

    @staticmethod
    def set_from_dict(cal_client, cc):
        resp = CalibrationConstant.get_by_uk(cal_client,
                                             cc['calibration_id'],
                                             cc['detector_type_id'],
                                             cc['condition_id'])

        if resp['success']:
            logging.debug('Got already existent calibration_constant')
        else:
            logging.debug('calibration_constant does not exist')
            resp = CalibrationConstant.create_from_dict(cal_client, cc)

        return resp

    @staticmethod
    def delete_by_id(cal_client, cc_id):
        resp = cal_client.delete_calibration_constant_api(cc_id)
        Base.cal_debug(MODULE_NAME, DELETE, resp)

        return Base.format_response(
            resp, DELETE, HTTPStatus.NO_CONTENT, MODULE_NAME
        )

    @staticmethod
    def get_by_id(cal_client, calibration_constant_id):
        response = cal_client.get_calibration_constant_by_id_api(
            calibration_constant_id)

        Base.cal_debug(MODULE_NAME, 'get_by_id', response)
        return Base.format_response(response, GET, HTTPStatus.OK, MODULE_NAME)

    @staticmethod
    def get_all_by_name(cal_client, name, page=DEF_PAGE, page_size=DEF_PAGE_SIZE):
        response = cal_client.get_all_calibration_constants_by_name_api(name, page, page_size)

        Base.cal_debug(MODULE_NAME, 'get_all_by_name', response)
        return Base.format_response(response, GET, HTTPStatus.OK, MODULE_NAME)

    @staticmethod
    def get_by_name(cal_client, name):
        res = CalibrationConstant.get_all_by_name(cal_client, name)

        if res['success']:
            if res['data'] == []:
                resp_data = []
            else:
                resp_data = res['data'][0]

            res = {'success': res['success'],
                   'info': res['info'],
                   'app_info': res['app_info'],
                   'data': resp_data}

        return res

    @staticmethod
    def get_all_by_conditions(cal_client, calibration_id, detector_type_id,
                              condition_ids):
        res = cal_client.get_all_calibration_constants_by_conds_api(
            calibration_id, detector_type_id, condition_ids)

        Base.cal_debug(MODULE_NAME,
                       'cc >> get_all_calibration_constants_by_conds_api',
                       res)
        return Base.format_response(res, GET, HTTPStatus.OK, MODULE_NAME)

    @staticmethod
    def get_by_uk(cal_client, calibration_id, detector_type_id, condition_id):
        res = cal_client.get_calibration_constant_by_uk_api(calibration_id,
                                                            detector_type_id,
                                                            condition_id)

        Base.cal_debug(MODULE_NAME, 'cc >> get_by_uk', res)
        return Base.format_response(res, GET, HTTPStatus.OK, MODULE_NAME)

    def __get_resource(self):
        calibration_constant = {
            CALIBRATION_CONSTANT: {
                'name': self.name,
                'calibration_id': self.calibration_id,
                'detector_type_id': self.detector_type_id,
                'condition_id': self.condition_id,
                'flg_available': self.flg_available,
                'flg_auto_approve': self.flg_auto_approve,
                'description': self.description
            }
        }

        return calibration_constant



"""PhysicalDetectorUnitApiTest class"""

import unittest
from http import HTTPStatus

import pytest

from .api_base import ApiBase
from ..common.config_test import *
from ..common.generators import generate_unique_name


@pytest.mark.usefixtures('client_cls')
class PhysicalDetectorUnitApiTest(ApiBase, unittest.TestCase):
    def test_create_physical_detector_unit_api(self):
        __unique_name = generate_unique_name('PhysicalDetectorUnitApi')
        physical_detector_unit = {
            PHYSICAL_DETECTOR_UNIT: {
                'physical_name': __unique_name,
                'karabo_da': __unique_name,
                'virtual_device_name': __unique_name,
                'detector_type_id': -1,
                'detector_id': -1,
                'uuid': 1000000000,
                'flg_available': True,
                'description': 'desc 01'
            }
        }

        expect = physical_detector_unit[PHYSICAL_DETECTOR_UNIT]

        # Create new entry (should succeed)
        received = self.__create_entry_api(physical_detector_unit, expect)

        physical_dev_id = received['id']
        pdu_phy_name = received['physical_name']
        physical_dev_det_id = received['detector_id']
        physical_dev_krb_da = received['karabo_da']

        try:
            # Create duplicated entry (should throw an error)
            self.__create_error_entry_uk_api(physical_detector_unit)

            # Get entry by name
            self.__get_all_entries_by_name_api(pdu_phy_name, expect)

            # Get entry by detector_id
            self.__get_all_entries_by_detector_id_api(physical_dev_det_id,
                                                      expect)

            # Get entry by detector_id
            self.__get_all_entries_by_detector_and_krbda_api(
                physical_dev_det_id, physical_dev_krb_da, expect)

            # Get entry by ID
            self.__get_entry_by_id_api(physical_dev_id, expect)

            # Put entry information (update some fields should succeed)
            self.__update_entry_api(physical_dev_id, expect)

        finally:
            # Delete entry (should succeed)
            # (test purposes only to keep the DB clean)
            del_resp = self.cal_client.delete_physical_detector_unit_api(
                physical_dev_id
            )

        # Validate the response from deletion if nothing already error-ed
        self.get_and_validate_delete_entry_by_id(del_resp)

    def fields_validation(self, receive, expect):
        assert receive['physical_name'] == expect['physical_name']
        assert receive['karabo_da'] == expect['karabo_da']
        assert receive['uuid'] == expect['uuid']
        assert receive['virtual_device_name'] == expect['virtual_device_name']
        assert receive['detector_type_id'] == expect['detector_type_id']
        assert receive['detector_id'] == expect['detector_id']
        assert receive['flg_available'] == expect['flg_available']
        assert receive['description'] == expect['description']

    def __create_entry_api(self, entry_info, expect):
        resp = self.cal_client.create_physical_detector_unit_api(entry_info)
        receive = self.get_and_validate_create_entry(resp)
        self.fields_validation(receive, expect)
        return receive

    def __create_error_entry_uk_api(self, entry_info):
        resp = self.cal_client.create_physical_detector_unit_api(entry_info)
        receive = self.load_response_content(resp)

        expect = {'info': {'physical_name': ['has already been taken'],
                           'karabo_da': ['has already been taken'],
                           'uuid': ['has already been taken']}}
        assert receive == expect
        assert resp.status_code == HTTPStatus.UNPROCESSABLE_ENTITY

    def __update_entry_api(self, entry_id, expect):
        unique_name_upd = generate_unique_name('PhysicalDevApiUpd')
        physical_dev_upd = {
            PHYSICAL_DETECTOR_UNIT: {
                'physical_name': expect['physical_name'],
                'karabo_da': unique_name_upd,
                'virtual_device_name': unique_name_upd,
                'uuid': expect['uuid'],
                # 'detector_type_id': '-1',
                # 'detector_id': '-1',
                'flg_available': False,
                'description': 'desc 01 updated!!!'
            }
        }

        resp = self.cal_client.update_physical_detector_unit_api(
            entry_id, physical_dev_upd)
        receive = self.load_response_content(resp)

        # Add parameters not send to the update API
        physical_dev_upd[PHYSICAL_DETECTOR_UNIT]['detector_type_id'] = -1
        physical_dev_upd[PHYSICAL_DETECTOR_UNIT]['detector_id'] = -1
        expect_upd = physical_dev_upd[PHYSICAL_DETECTOR_UNIT]

        self.fields_validation(receive, expect_upd)
        assert resp.status_code == HTTPStatus.OK

        # assert expect['physical_name'] != expect_upd['physical_name']
        assert expect['karabo_da'] != expect_upd['karabo_da']
        assert expect['virtual_device_name'] != expect_upd[
            'virtual_device_name']
        assert expect['flg_available'] != expect_upd['flg_available']
        assert expect['description'] != expect_upd['description']

    def __get_all_entries_by_name_api(self, physical_name, expect,
                                      page=DEF_PAGE, page_size=DEF_PAGE_SIZE):
        resp = self.cal_client.get_all_physical_detector_units_by_physical_name_api(physical_name,
                                                                                    page, page_size)  # noqa
        receive = self.get_and_validate_all_entries_by_name(resp)
        self.fields_validation(receive, expect)

    def __get_all_entries_by_detector_id_api(self, detector_id, expect):
        resp = self.cal_client.get_all_physical_detector_units_by_detector_id_api(detector_id)  # noqa
        receive = self.get_and_validate_all_entries_by_name(resp)

        self.fields_validation(receive, expect)

    def __get_all_entries_by_detector_and_krbda_api(self, detector_id,
                                                    karabo_da, expect):
        resp = self.cal_client.get_all_physical_detector_units_by_det_and_krbda_api(detector_id, karabo_da)  # noqa
        receive = self.get_and_validate_all_entries_by_name(resp)
        self.fields_validation(receive, expect)

    def __get_entry_by_id_api(self, entry_id, expect):
        response = self.cal_client.get_physical_detector_unit_by_id_api(entry_id)  # noqa
        receive = self.get_and_validate_entry_by_id(response)
        self.fields_validation(receive, expect)


if __name__ == '__main__':
    unittest.main()



import pytest

from calibration_client import CalibrationClient
from .common.secrets import CLIENT_OAUTH2_INFO, BASE_API_URL


@pytest.fixture(scope='session')
def client():
    return CalibrationClient(
        client_id=CLIENT_OAUTH2_INFO['CLIENT_ID'],
        client_secret=CLIENT_OAUTH2_INFO['CLIENT_SECRET'],
        token_url=CLIENT_OAUTH2_INFO['TOKEN_URL'],
        refresh_url=CLIENT_OAUTH2_INFO['REFRESH_URL'],
        auth_url=CLIENT_OAUTH2_INFO['AUTH_URL'],
        scope=CLIENT_OAUTH2_INFO['SCOPE'],
        user_email=CLIENT_OAUTH2_INFO['EMAIL'],
        base_api_url=BASE_API_URL
    )


# Fixture for use with unittest class-style tests
@pytest.fixture(scope="class")
def client_cls(request, client):
    request.cls.cal_client = client



"""ConditionApiTest class"""

import unittest
from http import HTTPStatus

import pytest

from .api_base import ApiBase
from ..common.config_test import RESOURCE_NOT_FOUND


@pytest.mark.usefixtures('client_cls')
class ConditionApiTest(ApiBase, unittest.TestCase):
    __unique_name = 'CONDITION_TEST-1_DO_NOT_DELETE'

    def test_expected_condition(self):
        parameters_conditions_attr = [
            {
                'parameter_id': -1,
                'value': 2.5,
                'lower_deviation_value': 0.5,
                'upper_deviation_value': 0.5,
                'flg_available': True,
                'description': 'Created automatically via seed:seed_tests'
            },
            {
                'parameter_id': -2,
                'value': 1.5,
                'lower_deviation_value': 0.5,
                'upper_deviation_value': 0.5,
                'flg_available': True,
                'description': 'Created automatically via seed:seed_tests'
            }
        ]

        condition = {
            'condition': {
                'name': self.__unique_name,
                'flg_available': True,
                'description': 'Created automatically via seed:seed_tests',
                'parameters_conditions_attributes': parameters_conditions_attr
            }
        }

        expect = condition['condition']
        # expect_params_conditions = len(parameters_conditions_attr)

        response1 = self.cal_client.set_expected_condition(condition)
        receive = self.load_response_content(response1)

        #
        # This test cannot be done because this function can return
        # either one of the following messages:
        # 1) self.assert_eq_str(response_content['info'],
        #   'Condition was successfully created.')
        # 2) self.assert_eq_str(response_content['info'],
        #   'Condition was successfully found.')
        self.fields_validation(receive, expect)
        # assert response1.status_code == HTTPStatus.CREATED
        assert response1.status_code == HTTPStatus.OK

        response2 = self.cal_client.get_expected_condition(condition)
        receive = self.load_response_content(response2)

        self.fields_validation(receive, expect)
        assert response2.status_code == HTTPStatus.OK

    def test_possible_conditions_exact_values(self):
        parameters_conditions_attr = [
            {
                'parameter_id': -1,
                'value': 2.5,
                'lower_deviation_value': 0.5,
                'upper_deviation_value': 0.5,
                'flg_available': True,
                'description': 'Created automatically via seed:seed_tests'
            },
            {
                'parameter_id': -2,
                'value': 1.5,
                'lower_deviation_value': 0.5,
                'upper_deviation_value': 0.5,
                'flg_available': True,
                'description': 'Created automatically via seed:seed_tests'
            }
        ]

        condition = {
            'condition': {
                'name': self.__unique_name,
                'event_at': '2019-07-25 09:14:52',
                'flg_available': True,
                'description': 'Created automatically via seed:seed_tests',
                'parameters_conditions_attributes': parameters_conditions_attr
            }
        }

        expect = condition['condition']

        response = self.cal_client.get_possible_conditions(condition)
        receive = self.load_response_content(response)[0]

        assert response.status_code == HTTPStatus.OK
        self.fields_validation(receive, expect)

    def test_possible_conditions_near_values(self):
        parameters_conditions_attr = [
            {
                'parameter_id': -1,
                'value': 2.0,
                'lower_deviation_value': 0.5,
                'upper_deviation_value': 0.5,
                'flg_available': True,
                'description': 'Created automatically via seed:seed_tests'
            },
            {
                'parameter_id': -2,
                'value': 1.999,
                'lower_deviation_value': 0.5,
                'upper_deviation_value': 0.5,
                'flg_available': True,
                'description': 'Created automatically via seed:seed_tests'
            }
        ]

        condition = {
            'condition': {
                'name': self.__unique_name,
                'flg_available': True,
                'description': 'Created automatically via seed:seed_tests',
                'parameters_conditions_attributes': parameters_conditions_attr
            }
        }

        expect = condition['condition']
        # expect_params_conditions = len(parameters_conditions_attr)

        response = self.cal_client.get_possible_conditions(condition)
        receive = self.load_response_content(response)[0]

        self.fields_validation(receive, expect)
        assert response.status_code == HTTPStatus.OK

    def test_possible_conditions_outside_values(self):
        parameters_conditions_attr = [
            {
                'parameter_id': -1,
                'value': 2.0,
                'lower_deviation_value': 0.5,
                'upper_deviation_value': 0.5,
                'flg_available': True,
                'description': 'Created automatically via seed:seed_tests'
            },
            {
                'parameter_id': -2,
                'value': 2.000001,  # Value outside bunderies
                'lower_deviation_value': 0.5,
                'upper_deviation_value': 0.5,
                'flg_available': True,
                'description': 'Created automatically via seed:seed_tests'
            }
        ]

        condition = {
            'condition': {
                'name': self.__unique_name,
                'flg_available': 'true',
                'description': '',
                'parameters_conditions_attributes': parameters_conditions_attr
            }
        }

        response = self.cal_client.get_possible_conditions(condition)
        receive = self.load_response_content(response)

        expect = {'info': RESOURCE_NOT_FOUND}
        assert receive == expect
        assert response.status_code == HTTPStatus.NOT_FOUND

    def fields_validation(self, receive, expect):
        assert receive['name'] == expect['name']
        assert receive['flg_available'] == expect['flg_available']
        assert receive['description'] == expect['description']

        assert receive['num_parameters'] == len(
            expect['parameters_conditions_attributes']
        )


if __name__ == '__main__':
    unittest.main()
